@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  *
  *
- *	$Id: im_kbd.c,v 1.2 2004/10/11 11:09:19 seiichi Exp $
+ *	$Id: im_kbd.c,v 1.7 2005/01/25 20:42:35 seiichi Exp $
  */
 
 #include  <X11/keysym.h>	/* XK_xxx */
@@ -53,6 +53,8 @@ typedef enum  kbd_type
 	KBD_TYPE_ARABIC ,
 	KBD_TYPE_ISCII ,
 
+	KBD_TYPE_MAX
+
 }  kbd_type_t ;
 
 typedef enum  kbd_mode
@@ -62,6 +64,8 @@ typedef enum  kbd_mode
 	KBD_MODE_ISCII_INSCRIPT ,
 	KBD_MODE_ISCII_PHONETIC ,
 
+	KBD_MODE_MAX
+	
 }  kbd_mode_t ;
 
 typedef struct im_kbd
@@ -86,7 +90,7 @@ static int  initialized = 0 ;
 static mkf_parser_t *  parser_ascii = NULL ;
 
 /* mlterm internal symbols */
-static x_im_export_syms_t *  mlterm_syms = NULL ;
+static x_im_export_syms_t *  syms = NULL ;
 
 static u_char *  arabic_conv_tbl[] =
 {
@@ -199,7 +203,7 @@ delete(
 
 	if( kbd->keymap)
 	{
-		(*mlterm_syms->ml_iscii_keymap_delete)( kbd->keymap) ;
+		(*syms->ml_iscii_keymap_delete)( kbd->keymap) ;
 	}
 
 	if( kbd->parser)
@@ -328,9 +332,9 @@ key_event_iscii(
 		return  1 ;
 	}
 
-	len = (*mlterm_syms->ml_convert_ascii_to_iscii)( kbd->keymap ,
-							 buf , sizeof( buf) ,
-							 &key_char , 1) ;
+	len = (*syms->ml_convert_ascii_to_iscii)( kbd->keymap ,
+						  buf , sizeof( buf) ,
+						  &key_char , 1) ;
 
 	(*kbd->im.listener->write_to_term)( kbd->im.listener->self ,
 					    buf , len) ;
@@ -362,7 +366,7 @@ switch_mode(
 	{
 		if( kbd->keymap)
 		{
-			(*mlterm_syms->ml_iscii_keymap_delete)( kbd->keymap) ;
+			(*syms->ml_iscii_keymap_delete)( kbd->keymap) ;
 			kbd->keymap = NULL ;
 		}
 
@@ -371,7 +375,7 @@ switch_mode(
 
 		if( kbd->mode == KBD_MODE_ASCII)
 		{
-			kbd->keymap = (*mlterm_syms->ml_iscii_keymap_new)( 1) ;
+			kbd->keymap = (*syms->ml_iscii_keymap_new)( 1) ;
 			kbd->mode = KBD_MODE_ISCII_INSCRIPT ;
 
 		#ifdef  IM_KBD_DEBUG
@@ -381,7 +385,7 @@ switch_mode(
 		}
 		else if( kbd->mode == KBD_MODE_ISCII_INSCRIPT)
 		{
-			kbd->keymap = (*mlterm_syms->ml_iscii_keymap_new)( 0) ;
+			kbd->keymap = (*syms->ml_iscii_keymap_new)( 0) ;
 			kbd->mode = KBD_MODE_ISCII_PHONETIC ;
 
 		#ifdef  IM_KBD_DEBUG
@@ -426,7 +430,7 @@ switch_mode(
 
 		if( kbd->im.stat_screen == NULL)
 		{
-			if( ! ( kbd->im.stat_screen = (*mlterm_syms->x_im_status_screen_new)(
+			if( ! ( kbd->im.stat_screen = (*syms->x_im_status_screen_new)(
 						(*kbd->im.listener->get_win_man)(kbd->im.listener->self) ,
 						(*kbd->im.listener->get_font_man)(kbd->im.listener->self) ,
 						(*kbd->im.listener->get_color_man)(kbd->im.listener->self) ,
@@ -500,15 +504,15 @@ unfocused(
 /* --- global functions --- */
 
 x_im_t *
-im_new(
+im_kbd_new(
 	u_int64_t  magic ,
 	ml_char_encoding_t  term_encoding ,
 	x_im_export_syms_t *  export_syms ,
-	char *  opts
+	char *  opts ,
+	u_int  mod_ignore_mask
 	)
 {
 	im_kbd_t *  kbd ;
-	void * hold_handler;
 
 	if( magic != (u_int64_t) IM_API_COMPAT_CHECK_MAGIC)
 	{
@@ -519,9 +523,9 @@ im_new(
 
 	if( ! initialized)
 	{
-		mlterm_syms = export_syms ;
+		syms = export_syms ;
 
-		if( ! ( parser_ascii = (*mlterm_syms->ml_parser_new)( ML_ISO8859_1)))
+		if( ! ( parser_ascii = (*syms->ml_parser_new)( ML_ISO8859_1)))
 		{
 			return  NULL ;
 		}
@@ -562,7 +566,7 @@ im_new(
 		}
 	}
 
-	if( ! ( kbd->conv = (*mlterm_syms->ml_conv_new)( term_encoding)))
+	if( ! ( kbd->conv = (*syms->ml_conv_new)( term_encoding)))
 	{
 		goto  error ;
 	}
@@ -611,13 +615,12 @@ error:
 /* --- API for external tools --- */
 
 im_info_t *
-im_get_info(
+im_kbd_get_info(
 	char *  locale ,
 	char *  encoding
 )
 {
 	im_info_t *  result ;
-	int  i ;
 
 	if( ! ( result = malloc( sizeof( im_info_t))))
 	{
