@@ -28,7 +28,7 @@
  * SUCH DAMAGE.
  *
  *
- *	$Id: im_uim.c,v 1.21 2004/10/21 21:02:34 seiichi Exp $
+ *	$Id: im_uim.c,v 1.37 2005/03/02 14:13:46 seiichi Exp $
  */
 
 #include  <uim.h>
@@ -42,18 +42,17 @@
 #include  <kiklib/kik_str.h>	/* kik_str_alloca_dup kik_str_sep kik_snprintf*/
 #include  <kiklib/kik_locale.h>	/* kik_get_locale */
 #include  <kiklib/kik_list.h>
-#include  <signal.h>
 
 #include  <x_im.h>
 #include  "../im_common.h"
 #include  "../im_info.h"
 
 #if  0
-#define IM_UIM_DEBUG 1
+#define  IM_UIM_DEBUG  1
 #endif
 
 #ifndef  UIM_0_4_4_OR_LATER
-#define  IM_UIM_COMPAT_0_3_8 1
+#define  UIM_0_3_8_COMPAT  1
 /* see http://www.freedesktop.org/pipermail/uim/2004-June/000383.html */
 #endif
 
@@ -73,15 +72,17 @@ typedef struct im_uim
 
 	ml_char_encoding_t  term_encoding ;
 
-	/* native encoding of conversion engine */
-	char *  encoding_name ;
-
-	char *  engine_name ;
+	char *  encoding_name ;		/* encoding of conversion engine */
 
 	mkf_parser_t *  parser_uim ;	/* for uim encoding  */
 	mkf_parser_t *  parser_term ;	/* for term encoding  */
 	mkf_conv_t *  conv ;
 
+	u_int  pressing_mod_key ;
+	u_int  mod_ignore_mask ;
+
+	u_int  cand_limit ;
+
 }  im_uim_t ;
 
 KIK_LIST_TYPEDEF( im_uim_t) ;
@@ -94,10 +95,9 @@ static KIK_LIST( im_uim_t)  uim_list = NULL ;
 static int  ref_count = 0 ;
 static int  initialized = 0 ;
 static int  helper_fd = -1 ;
-static im_uim_t *  last_focused_uim = NULL ;
-
-/* mlterm internal symbols */
-static x_im_export_syms_t *  mlterm_syms = NULL ;
+static im_uim_t *  focused_uim = NULL ;
+static x_im_export_syms_t *  syms = NULL ; /* mlterm internal symbols */
+static int  mod_key_debug = 0 ;
 
 
 /* --- static functions --- */
@@ -155,12 +155,12 @@ xksym_to_ukey(
 	)
 {
 	/* Latin 1 */
-	if (XK_space <= ksym && ksym <= XK_asciitilde)
+	if( XK_space <= ksym && ksym <= XK_asciitilde)
 	{
 		return  ksym ;
 	}
 
-	switch (ksym)
+	switch( ksym)
 	{
 	/* TTY Functions */
 	case  XK_BackSpace:
@@ -195,7 +195,7 @@ xksym_to_ukey(
 	case  XK_Down:
 		return  UKey_Down ;
 	case  XK_Prior:
-		return  XK_Prior ;
+		return  UKey_Prior ;
 	case  XK_Next:
 		return  UKey_Next ;
 	case  XK_End:
@@ -345,11 +345,7 @@ xksym_to_ukey(
 static void
 helper_disconnected( void)
 {
-#ifdef  IM_UIM_DEBUG
-	kik_debug_printf("helper was disconnected\n");
-#endif
-
-	(*mlterm_syms->x_term_manager_remove_fd)( helper_fd) ;
+	(*syms->x_term_manager_remove_fd)( helper_fd) ;
 
 	helper_fd = -1 ;
 }
@@ -365,37 +361,38 @@ prop_list_update(
 	int  len ;
 
 #ifdef  IM_UIM_DEBUG
-	kik_debug_printf("prop_list_update(), str: %s\n", str);
+	kik_debug_printf( KIK_DEBUG_TAG " str: %s\n", str);
 #endif
 
 	uim = (im_uim_t*) p ;
 
-#ifdef  IM_UIM_COMPAT_0_3_8
-	if( last_focused_uim->context == p)
+#ifdef  UIM_0_3_8_COMPAT
+	if( focused_uim->context == p)
 	{
-		uim = (im_uim_t*) last_focused_uim ;
+		uim = (im_uim_t*) focused_uim ;
 	}
 #endif
 
-#define  PROP_LIST_FORMAT			\
-	"prop_list_update\ncharset=%s\n%s"	\
-	"branch\t%s\t%s\n"
+	if( focused_uim != uim)
+	{
+		return ;
+	}
 
-	len = strlen(PROP_LIST_FORMAT) + strlen( uim->encoding_name) +
-	      strlen( str) + (strlen(uim->engine_name) * 2) + 1 ;
+#define  PROP_LIST_FORMAT  "prop_list_update\ncharset=%s\n%s"
+
+	len = strlen( PROP_LIST_FORMAT) + strlen( uim->encoding_name) +
+	      strlen( str) + 1 ;
 
 	if( len > sizeof( buf))
 	{
 	#ifdef  DEBUG
-		kik_warn_printf( "property list string is too long.");
+		kik_warn_printf( KIK_DEBUG_TAG " property list string is too long.");
 	#endif
-
 		return ;
 	}
 
-	kik_snprintf( buf , sizeof(buf) , PROP_LIST_FORMAT ,
-		      uim->encoding_name , str ,
-		      uim->engine_name , uim->engine_name) ;
+	kik_snprintf( buf , sizeof( buf) , PROP_LIST_FORMAT ,
+		      uim->encoding_name , str) ;
 
 	uim_helper_send_message( helper_fd , buf) ;
 
@@ -413,22 +410,27 @@ prop_label_update(
 	int  len ;
 
 #ifdef  IM_UIM_DEBUG
-	kik_debug_printf("prop_label_update(), str: %s\n", str);
+	kik_debug_printf( KIK_DEBUG_TAG " prop_label_update(), str: %s\n", str);
 #endif
 
 	uim = (im_uim_t*) p ;
 
-#ifdef  IM_UIM_COMPAT_0_3_8
-	if( last_focused_uim->context == p)
+#ifdef  UIM_0_3_8_COMPAT
+	if( focused_uim->context == p)
 	{
-		uim = (im_uim_t*) last_focused_uim ;
+		uim = focused_uim ;
 	}
 #endif
 
-#define  PROP_LABEL_FORMAT "prop_label_update\ncharset=%s\n%s%s\t%s\n"
+	if( focused_uim != uim)
+	{
+		return ;
+	}
+
+#define  PROP_LABEL_FORMAT "prop_label_update\ncharset=%s\n%s"
 
 	len = strlen(PROP_LABEL_FORMAT) + strlen( uim->encoding_name) +
-	      strlen( str) + (strlen(uim->engine_name) * 2) + 1 ;
+	      strlen( str) + 1 ;
 
 	if( len > sizeof( buf))
 	{
@@ -440,8 +442,7 @@ prop_label_update(
 	}
 
 	kik_snprintf( buf , sizeof(buf) , PROP_LABEL_FORMAT ,
-		      uim->encoding_name , str ,
-		      uim->engine_name , uim->engine_name) ;
+		      uim->encoding_name , str) ;
 
 	uim_helper_send_message( helper_fd , buf) ;
 
@@ -458,33 +459,32 @@ commit(
 	im_uim_t *  uim ;
 	u_char  conv_buf[256] ;
 	size_t  filled_len ;
+	size_t  len ;
 
 #ifdef  IM_UIM_DEBUG
-	kik_debug_printf("commit(), str: %s\n", str);
+	kik_debug_printf( KIK_DEBUG_TAG "str: %s\n", str);
 #endif
 
 	uim = (im_uim_t*) p ;
 
+	len = strlen( str) ;
+
 	if( ! NEED_TO_CONV(uim))
 	{
 		(*uim->im.listener->write_to_term)( uim->im.listener->self ,
-						    (u_char*)str ,
-						    strlen( str)) ;
+						    (u_char*)str , len) ;
 
 		return ;
 	}
 
 	(*uim->parser_uim->init)( uim->parser_uim) ;
-	(*uim->parser_uim->set_str)( uim->parser_uim ,
-				     (u_char*)str ,
-				     strlen(str)) ;
+	(*uim->parser_uim->set_str)( uim->parser_uim , (u_char*)str , len) ;
 
 	(*uim->conv->init)( uim->conv) ;
 
 	while( ! uim->parser_uim->is_eos)
 	{
-		filled_len = (*uim->conv->convert)( uim->conv ,
-						    conv_buf ,
+		filled_len = (*uim->conv->convert)( uim->conv , conv_buf ,
 						    sizeof( conv_buf) ,
 						    uim->parser_uim) ;
 
@@ -495,8 +495,7 @@ commit(
 		}
 
 		(*uim->im.listener->write_to_term)( uim->im.listener->self ,
-						    conv_buf ,
-						    filled_len) ;
+						    conv_buf , filled_len) ;
 	}
 }
 
@@ -506,7 +505,6 @@ preedit_clear(
 	)
 {
 	im_uim_t *  uim ;
-	int  i ;
 
 #ifdef  IM_UIM_DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG "\n") ;
@@ -516,8 +514,8 @@ preedit_clear(
 
 	if( uim->im.preedit.chars)
 	{
-		(*mlterm_syms->ml_str_delete)( uim->im.preedit.chars ,
-					       uim->im.preedit.num_of_chars) ;
+		(*syms->ml_str_delete)( uim->im.preedit.chars ,
+					uim->im.preedit.num_of_chars) ;
 		uim->im.preedit.chars = NULL ;
 	}
 
@@ -542,11 +540,10 @@ preedit_pushback(
 	ml_color_t  bg_color = ML_BG_COLOR ;
 	int  is_underline = 0 ;
 	u_int  count = 0 ;
+	size_t  len ;
 
 #ifdef  IM_UIM_DEBUG
-	kik_debug_printf( KIK_DEBUG_TAG
-			  " attr: %d, _str:%s, length:%d\n" ,
-			  attr , _str, strlen( _str)) ;
+	kik_debug_printf( KIK_DEBUG_TAG " attr: %d, _str:%s, length:%d\n" , attr , _str, strlen( _str)) ;
 #endif
 
 	uim = (im_uim_t*) ptr ;
@@ -556,7 +553,7 @@ preedit_pushback(
 		uim->im.preedit.cursor_offset = uim->im.preedit.filled_len ;
 	}
 
-	if( ! strlen( _str))
+	if( ! ( len = strlen( _str)))
 	{
 		return ;
 	}
@@ -581,25 +578,24 @@ preedit_pushback(
 		/* uim encoding -> term encoding */
 		(*uim->parser_uim->init)( uim->parser_uim) ;
 		if( ! (im_convert_encoding( uim->parser_uim , uim->conv ,
-					    (u_char*)_str , &str ,
-					    strlen( _str) + 1)))
+					    (u_char*)_str , &str , len + 1)))
 		{
 			return ;
 		}
 	}
 	else
 	{
-		str = (u_char*)_str ;
+		str = (u_char*) _str ;
 	}
 
+	len = strlen( str) ;
+
 	/*
 	 * count number of characters to re-allocate im.preedit.chars
 	 */
 
 	(*uim->parser_term->init)( uim->parser_term) ;
-	(*uim->parser_term->set_str)( uim->parser_term ,
-				      (u_char*) str ,
-				      strlen( str)) ;
+	(*uim->parser_term->set_str)( uim->parser_term , (u_char*) str , len) ;
 
 	while( (*uim->parser_term->next_char)( uim->parser_term , &ch))
 	{
@@ -616,14 +612,16 @@ preedit_pushback(
 			kik_warn_printf( KIK_DEBUG_TAG " realloc failed.\n") ;
 		#endif
 
-			(*mlterm_syms->ml_str_delete)(
-						uim->im.preedit.chars ,
+			(*syms->ml_str_delete)( uim->im.preedit.chars ,
 						uim->im.preedit.num_of_chars) ;
 			uim->im.preedit.chars = NULL ;
 			uim->im.preedit.num_of_chars = 0 ;
 			uim->im.preedit.filled_len = 0 ;
 
-			free( str) ;
+			if( NEED_TO_CONV(uim))
+			{
+				free( str) ;
+			}
 
 			return ;
 		}
@@ -637,12 +635,10 @@ preedit_pushback(
 	 */
 
 	p = &uim->im.preedit.chars[uim->im.preedit.filled_len];
-	(*mlterm_syms->ml_str_init)( p , count);
+	(*syms->ml_str_init)( p , count);
 
 	(*uim->parser_term->init)( uim->parser_term) ;
-	(*uim->parser_term->set_str)( uim->parser_term ,
-				      (u_char*)str ,
-				      strlen( str)) ;
+	(*uim->parser_term->set_str)( uim->parser_term , (u_char*)str , len) ;
 
 	count = 0 ;
 
@@ -671,13 +667,11 @@ preedit_pushback(
 
 		if( is_comb)
 		{
-			if( (*mlterm_syms->ml_char_combine)( p - 1 , ch.ch ,
-							     ch.size , ch.cs ,
-							     is_biwidth ,
-							     is_comb ,
-							     fg_color ,
-							     bg_color ,
-							     0 , is_underline))
+			if( (*syms->ml_char_combine)( p - 1 , ch.ch ,
+						      ch.size , ch.cs ,
+						      is_biwidth , is_comb ,
+						      fg_color , bg_color ,
+						      0 , is_underline))
 			{
 				continue;
 			}
@@ -687,15 +681,15 @@ preedit_pushback(
 			 */
 		}
 
-		if( (*mlterm_syms->ml_is_msb_set)( ch.cs))
+		if( (*syms->ml_is_msb_set)( ch.cs))
 		{
 			SET_MSB( ch.ch[0]) ;
 		}
 
-		(*mlterm_syms->ml_char_set)( p , ch.ch , ch.size , ch.cs ,
-					     is_biwidth , is_comb ,
-					     fg_color , bg_color ,
-					     0 , is_underline) ;
+		(*syms->ml_char_set)( p , ch.ch , ch.size , ch.cs ,
+				      is_biwidth , is_comb ,
+				      fg_color , bg_color ,
+				      0 , is_underline) ;
 
 		p++ ;
 		uim->im.preedit.filled_len++;
@@ -732,15 +726,10 @@ candidate_selected(
 	u_int  index
 	)
 {
-	im_uim_t *  uim ;
-
 #ifdef  IM_UIM_DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " index : %d\n" , index) ;
 #endif
-
-	uim = (im_uim_t*) p ;
-
-	uim_set_candidate_index( uim->context , index) ;
+	uim_set_candidate_index( ((im_uim_t*)p)->context , index) ;
 }
 
 
@@ -768,19 +757,17 @@ candidate_activate(
 
 	if( uim->im.cand_screen == NULL)
 	{
-		int  is_vertical ;
-		u_int  line_height ;
-
 		(*uim->im.listener->get_spot)( uim->im.listener->self ,
 					       uim->im.preedit.chars ,
 					       uim->im.preedit.segment_offset ,
 					       &x , &y) ;
 
-		if( ! ( uim->im.cand_screen = (*mlterm_syms->x_im_candidate_screen_new)(
+		if( ! ( uim->im.cand_screen = (*syms->x_im_candidate_screen_new)(
 				(*uim->im.listener->get_win_man)(uim->im.listener->self) ,
 				(*uim->im.listener->get_font_man)(uim->im.listener->self) ,
 				(*uim->im.listener->get_color_man)(uim->im.listener->self) ,
 				(*uim->im.listener->is_vertical)(uim->im.listener->self) ,
+				1 ,
 				(*uim->im.listener->get_line_height)(uim->im.listener->self) ,
 				x , y)))
 		{
@@ -795,7 +782,7 @@ candidate_activate(
 		uim->im.cand_screen->listener.selected = candidate_selected ;
 	}
 
-	if( ! (*uim->im.cand_screen->init)( uim->im.cand_screen , num , 10))
+	if( ! (*uim->im.cand_screen->init)( uim->im.cand_screen , num , limit))
 	{
 		(*uim->im.cand_screen->delete)( uim->im.cand_screen) ;
 		uim->im.cand_screen = NULL ;
@@ -824,10 +811,9 @@ candidate_activate(
 			if( im_convert_encoding( uim->parser_uim , uim->conv ,
 						 _p , &p , strlen( _p) + 1))
 			{
-				(*uim->im.cand_screen->set)(
-							uim->im.cand_screen ,
-							uim->parser_term ,
-							p , i) ;
+				(*uim->im.cand_screen->set)( uim->im.cand_screen ,
+							     uim->parser_term ,
+							     p , i) ;
 				free( p) ;
 			}
 		}
@@ -842,6 +828,7 @@ candidate_activate(
 	}
 
 	(*uim->im.cand_screen->select)( uim->im.cand_screen , 0) ;
+	uim->cand_limit = limit ;
 }
 
 static void
@@ -860,8 +847,7 @@ candidate_select(
 
 	if( uim->im.cand_screen)
 	{
-			(*uim->im.cand_screen->select)( uim->im.cand_screen ,
-							index) ;
+		(*uim->im.cand_screen->select)( uim->im.cand_screen , index) ;
 	}
 }
 
@@ -872,14 +858,49 @@ candidate_shift_page(
 	)
 {
 	im_uim_t *  uim ;
+	int index ;
 
 #ifdef  IM_UIM_DEBUG
-	kik_debug_printf( KIK_DEBUG_TAG " direction: %d\n", direction) ;
+	kik_debug_printf( KIK_DEBUG_TAG " direction: %s\n", direction ? "next" : "prev") ;
 #endif
 
 	uim = (im_uim_t*) p ;
 
-	/* XXX: not implemented yet */
+	if( ! uim->im.cand_screen)
+	{
+		return ;
+	}
+
+	index = (int) uim->im.cand_screen->index ;
+
+	if( ! direction && index < uim->cand_limit)
+	{
+		/* top page -> last page */
+		index = (uim->im.cand_screen->num_of_candidates / uim->cand_limit) * uim->cand_limit + index ;
+	}
+	else if( direction &&
+		 ((index / uim->cand_limit) + 1) * uim->cand_limit > uim->im.cand_screen->num_of_candidates)
+	{
+		/* last page -> top page */
+		index = index % uim->cand_limit ;
+	}
+	else
+	{
+		/* shift page according to the direction */
+		index += (direction ? uim->cand_limit : -(uim->cand_limit)) ;
+	}
+
+	if( index < 0)
+	{
+		index = 0 ;
+	}
+	else if( index >= uim->im.cand_screen->num_of_candidates)
+	{
+		index = uim->im.cand_screen->num_of_candidates - 1 ;
+	}
+
+	(*uim->im.cand_screen->select)( uim->im.cand_screen , index) ;
+	uim_set_candidate_index( uim->context , index) ;
 }
 
 static void
@@ -915,9 +936,9 @@ delete(
 
 	uim = (im_uim_t*) im ;
 
-	if( last_focused_uim == uim)
+	if( focused_uim == uim)
 	{
-		last_focused_uim = NULL ;
+		focused_uim = NULL ;
 	}
 
 	if( uim->parser_uim)
@@ -932,14 +953,9 @@ delete(
 		(*uim->conv->delete)( uim->conv) ;
 	}
 
-	if( uim->engine_name)
-	{
-		free( uim->engine_name) ;
-	}
-
 	uim_release_context( uim->context) ;
 
-	ref_count-- ;
+	ref_count -- ;
 
 #ifdef  IM_UIM_DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " An object was deleted. ref_count: %d\n", ref_count) ;
@@ -952,7 +968,7 @@ delete(
 	if( ref_count == 0 && initialized)
 	{
 		uim_helper_close_client_fd( helper_fd) ;
-		(*mlterm_syms->x_term_manager_remove_fd)( helper_fd) ;
+		(*syms->x_term_manager_remove_fd)( helper_fd) ;
 		helper_fd = -1 ;
 
 		uim_quit() ;
@@ -996,6 +1012,49 @@ key_event(
 
 	uim = (im_uim_t*) im ;
 
+	if( mod_key_debug)
+	{
+		kik_msg_printf( ">>--------------------------------\n") ;
+		kik_msg_printf( ">>event->state    : %.8x\n" , event->state) ;
+		kik_msg_printf( ">>mod_ignore_mask : %.8x\n" , uim->mod_ignore_mask) ;
+		kik_msg_printf( ">>ksym            : %.8x\n" , ksym) ;
+	}
+
+	if( ! ( event->state & uim->mod_ignore_mask))
+	{
+		uim->pressing_mod_key = 0 ;
+	}
+
+	switch( ksym)
+	{
+	case XK_Shift_L:
+	case XK_Shift_R:
+		uim->pressing_mod_key |= UMod_Shift ;
+		break ;
+	case XK_Control_L:
+	case XK_Control_R:
+		uim->pressing_mod_key |= UMod_Control ;
+		break ;
+	case XK_Alt_L:
+	case XK_Alt_R:
+		uim->pressing_mod_key |= UMod_Alt ;
+		break ;
+	case XK_Meta_L:
+	case XK_Meta_R:
+		uim->pressing_mod_key |= UMod_Meta ;
+		break ;
+	case XK_Super_L:
+	case XK_Super_R:
+		uim->pressing_mod_key |= UMod_Super ;
+		break ;
+	case XK_Hyper_L:
+	case XK_Hyper_R:
+		uim->pressing_mod_key |= UMod_Hyper ;
+		break ;
+	default:
+		break ;
+	}
+
 	(*uim->im.listener->compare_key_state_with_modmap)(
 							uim->im.listener->self ,
 							event->state ,
@@ -1004,12 +1063,37 @@ key_event(
 							&is_meta , &is_super ,
 							&is_hyper) ;
 
-	if( is_shift) state |= UKey_Shift;
-	if( is_ctl)   state |= UKey_Control;
-	if( is_alt)   state |= UMod_Alt ;
-	if( is_meta)  state |= UMod_Meta ;
-	if( is_super) state |= UMod_Super ;
-	if( is_hyper) state |= UMod_Hyper ;
+	if( is_shift && (uim->pressing_mod_key & UMod_Shift))
+	{
+		state |= UMod_Shift ;
+	}
+	if( is_ctl && (uim->pressing_mod_key & UMod_Control))
+	{
+		state |= UMod_Control ;
+	}
+	if( is_alt && (uim->pressing_mod_key & UMod_Alt))
+	{
+		state |= UMod_Alt ;
+	}
+	if( is_meta && (uim->pressing_mod_key & UMod_Meta))
+	{
+		state |= UMod_Meta ;
+	}
+	if( is_super && (uim->pressing_mod_key & UMod_Super))
+	{
+		state |= UMod_Super ;
+	}
+	if( is_hyper && (uim->pressing_mod_key & UMod_Hyper))
+	{
+		state |= UMod_Hyper ;
+	}
+
+	if( mod_key_debug)
+	{
+		kik_msg_printf( ">>pressing_mod_key: %.8x\n" , uim->pressing_mod_key) ;
+		kik_msg_printf( ">>state           : %.8x\n" , state) ;
+		kik_msg_printf( ">>--------------------------------\n") ;
+	}
 
 	key = xksym_to_ukey(ksym) ;
 
@@ -1036,17 +1120,19 @@ focused(
 
 	uim =  (im_uim_t*)  im ;
 
-	last_focused_uim = uim ;
+	uim_helper_client_focus_in( uim->context) ;
+
+	focused_uim = uim ;
 
 	uim_prop_list_update( uim->context) ;
 	uim_prop_label_update( uim->context) ;
 
-	uim_helper_client_focus_in( uim->context) ;
-
 	if( uim->im.cand_screen)
 	{
 		(*uim->im.cand_screen->show)( uim->im.cand_screen) ;
 	}
+
+	uim->pressing_mod_key = 0 ;
 }
 
 static void
@@ -1064,6 +1150,8 @@ unfocused(
 	{
 		(*uim->im.cand_screen->hide)( uim->im.cand_screen) ;
 	}
+
+	uim->pressing_mod_key = 0 ;
 }
 
 
@@ -1072,12 +1160,9 @@ unfocused(
  */
 
 static void
-helper_send_imlist(
-	im_uim_t *  last_focused_uim
-	)
+helper_send_imlist(void)
 {
 #ifdef  UIM_0_4_4_OR_LATER
-	im_uim_t *  uim ;
 	const char *  selected_name ;
 	const char *  name ;
 	const char *  lang ;
@@ -1087,38 +1172,24 @@ helper_send_imlist(
 	u_int  len = 0 ;
 	u_int  filled_len = 0 ;
 
-	if( last_focused_uim)
+	if( ! focused_uim)
 	{
-		uim = last_focused_uim ;
-	}
-	else
-	{
-		KIK_ITERATOR( im_uim_t)  iterator = NULL ;
-
-		if( uim_list == NULL)
-		{
-			return ;
-		}
-
-		if( ! (iterator = kik_list_first( uim_list)))
-		{
-			uim = kik_iterator_indirect( iterator) ;
-		}
+		return ;
 	}
 
 #define  HEADER_FORMAT  "im_list\ncharset=%s\n"
 
-	len += strlen( HEADER_FORMAT) + strlen( uim->encoding_name) ;
+	len += strlen( HEADER_FORMAT) + strlen( focused_uim->encoding_name) ;
 
-	selected_name = uim_get_current_im_name( uim->context) ;
+	selected_name = uim_get_current_im_name( focused_uim->context) ;
 	len += strlen( selected_name) ;
 	len += strlen( "selected") ;
 
-	for( i = 0 ; i < uim_get_nr_im( uim->context) ; i++)
+	for( i = 0 ; i < uim_get_nr_im( focused_uim->context) ; i++)
 	{
-		name = uim_get_im_name( uim->context , i) ;
-		lang = uim_get_im_language( uim->context , i) ;
-		dsc = uim_get_im_short_desc( uim->context , i) ;
+		name = uim_get_im_name( focused_uim->context , i) ;
+		lang = uim_get_im_language( focused_uim->context , i) ;
+		dsc = uim_get_im_short_desc( focused_uim->context , i) ;
 
 		len += name ? strlen( name) : 0 ;
 		len += lang ? strlen( lang) : 0 ;
@@ -1136,16 +1207,16 @@ helper_send_imlist(
 		return ;
 	}
 
-	filled_len = kik_snprintf( buf , len ,
-				   HEADER_FORMAT , uim->encoding_name) ;
+	filled_len = kik_snprintf( buf , len , HEADER_FORMAT ,
+				   focused_uim->encoding_name) ;
 
 #undef  HEADER_FORMAT
 
-	for( i = 0 ; i < uim_get_nr_im( uim->context) ; i++)
+	for( i = 0 ; i < uim_get_nr_im( focused_uim->context) ; i++)
 	{
-		name = uim_get_im_name( uim->context , i) ;
-		lang = uim_get_im_language( uim->context , i) ;
-		dsc = uim_get_im_short_desc( uim->context , i) ;
+		name = uim_get_im_name( focused_uim->context , i) ;
+		lang = uim_get_im_language( focused_uim->context , i) ;
+		dsc = uim_get_im_short_desc( focused_uim->context , i) ;
 
 		filled_len += kik_snprintf( &buf[filled_len] ,
 					    len - filled_len ,
@@ -1153,8 +1224,7 @@ helper_send_imlist(
 					    name ? name : "" ,
 					    lang ? lang : "" ,
 					    dsc ? dsc : "" ,
-					    strcmp( name , selected_name) == 0 ?
-							"selected" : "") ;
+					    strcmp( name , selected_name) == 0 ?  "selected" : "") ;
 	}
 
 #ifdef  IM_UIM_DEBUG
@@ -1162,13 +1232,11 @@ helper_send_imlist(
 #endif
 
 	uim_helper_send_message( helper_fd , buf) ;
-
 #endif
 }
 
 static void
 helper_im_changed(
-	im_uim_t *  last_focused_uim ,
 	char *  request ,
 	char *  engine_name
 	)
@@ -1196,17 +1264,16 @@ helper_im_changed(
 
 	if( strcmp( request , "im_change_this_text_area_only") == 0)
 	{
-		if( last_focused_uim)
+		if( focused_uim)
 		{
-			(*last_focused_uim->im.listener->im_changed)(
-					last_focused_uim->im.listener->self ,
-					buf) ;
+			(*focused_uim->im.listener->im_changed)(
+						focused_uim->im.listener->self ,
+						buf) ;
 		}
 	}
 	else if( strcmp( request , "im_change_whole_desktop") == 0 ||
 		 strcmp( request , "im_change_this_application_only") == 0)
 	{
-		im_uim_t *  uim ;
 		KIK_ITERATOR( im_uim_t)  iterator = NULL ;
 
 		iterator = kik_list_first( uim_list) ;
@@ -1221,24 +1288,116 @@ helper_im_changed(
 			}
 			else
 			{
+				im_uim_t *  uim ;
 				uim = kik_iterator_indirect( iterator) ;
 				(*uim->im.listener->im_changed)(
-						uim->im.listener->self , buf) ;
+							uim->im.listener->self ,
+							buf) ;
 			}
 
 			iterator = kik_iterator_next( iterator) ;
 		}
 	}
-
-#endif
+#endif  /* UIM_0_4_4_OR_LATER */
 }
 
 static void
-helper_read_handler( void)
+helper_update_custom(
+	char *  custom ,
+	char *  value
+	)
+{
+#ifdef  UIM_0_4_6_OR_LATER
+	KIK_ITERATOR( im_uim_t)  iterator = NULL ;
+
+	iterator = kik_list_first( uim_list) ;
+
+	while( iterator)
+	{
+		if( kik_iterator_indirect( iterator) == NULL)
+		{
+			kik_error_printf(
+				"iterator found, but it has no logs. "
+				"don't you cross over memory "
+				"boundaries anywhere?\n") ;
+		}
+		else
+		{
+			uim_prop_update_custom( kik_iterator_indirect( iterator)->context , custom , value) ;
+		}
+
+		iterator = kik_iterator_next( iterator) ;
+	}
+
+#endif  /* UIM_0_4_6_OR_LATER */
+}
+
+static void
+helper_commit_string(
+	u_char *  str	/* UTF-8? */
+	)
+{
+	mkf_parser_t *  parser_utf8 ;
+	mkf_conv_t *  conv ;
+	u_char  conv_buf[256] ;
+	size_t  filled_len ;
+
+	if( ! focused_uim)
+	{
+		return ;
+	}
+
+	if( focused_uim->term_encoding == ML_UTF8)
+	{
+		(*focused_uim->im.listener->write_to_term)(
+						focused_uim->im.listener->self ,
+						str , strlen( str)) ;
+		return ;
+	}
+
+	if( ! ( conv = (*syms->ml_conv_new)( focused_uim->term_encoding)))
+	{
+		return ;
+	}
+
+	if( ! ( parser_utf8 = (*syms->ml_parser_new)( ML_UTF8)))
+	{
+		(*conv->delete)( conv) ;
+		return ;
+	}
+
+	(*parser_utf8->init)( parser_utf8) ;
+	(*parser_utf8->set_str)( parser_utf8 , str , strlen( str)) ;
+
+	(*conv->init)( conv) ;
+
+	while( ! parser_utf8->is_eos)
+	{
+		filled_len = (*conv->convert)( conv , conv_buf ,
+					       sizeof( conv_buf) ,
+					       parser_utf8) ;
+
+		if( filled_len == 0)
+		{
+			/* finished converting */
+			break ;
+		}
+
+		(*focused_uim->im.listener->write_to_term)(
+						focused_uim->im.listener->self ,
+						conv_buf , filled_len) ;
+	}
+
+	(*parser_utf8->delete)( parser_utf8) ;
+	(*conv->delete)( conv) ;
+}
+
+static void
+helper_read_handler(void)
 {
 	char *  message ;
 
-	uim_helper_read_proc(helper_fd);
+	uim_helper_read_proc( helper_fd);
 
 	while( ( message = uim_helper_get_message()))
 	{
@@ -1254,35 +1413,45 @@ helper_read_handler( void)
 			if( strcmp( first_line , "prop_activate") == 0)
 			{
 				second_line = kik_str_sep( &message , "\n") ;
-
-				if( second_line && last_focused_uim)
+				if( second_line && focused_uim)
 				{
-					uim_prop_activate(
-						last_focused_uim->context ,
-						second_line) ;
+					uim_prop_activate( focused_uim->context , second_line) ;
 				}
 			}
 			else if( strcmp( first_line , "im_list_get") == 0)
 			{
-				if( last_focused_uim)
-				{
-					helper_send_imlist( last_focused_uim) ;
-				}
+				helper_send_imlist() ;
 			}
 			else if( strncmp( first_line , "im_change_" , 10) == 0)
 			{
-				second_line = kik_str_sep( &message , "\n") ;
-
-				if( second_line && last_focused_uim)
+				if( ( second_line = kik_str_sep( &message , "\n")))
 				{
-					helper_im_changed( last_focused_uim ,
-							   first_line ,
-							   second_line) ;
+					helper_im_changed( first_line , second_line) ;
 				}
 			}
+			else if( strcmp( first_line , "prop_update_custom") == 0)
+			{
+				if( ( second_line = kik_str_sep( &message , "\n")))
+				{
+					helper_update_custom( second_line , message) ;
+				}
+			}
+			else if( strcmp( first_line , "focus_in") == 0)
+			{
+				focused_uim = NULL ;
+			}
+			else if( strcmp( first_line , "commit_string") == 0)
+			{
+				if( ( second_line = kik_str_sep( &message , "\n")))
+				{
+					helper_commit_string( second_line) ;
+				}
+			}
+
+			message = first_line ; /* for free() */
 		}
 
-		free( first_line) ;
+		free( message) ;
 	}
 }
 
@@ -1290,17 +1459,17 @@ helper_read_handler( void)
 /* --- global functions --- */
 
 x_im_t *
-im_new(
+im_uim_new(
 	u_int64_t  magic ,
 	ml_char_encoding_t  term_encoding ,
 	x_im_export_syms_t *  export_syms ,
-	char *  engine
+	char *  engine ,
+	u_int  mod_ignore_mask
 	)
 {
 	im_uim_t *  uim ;
 	char *  encoding_name ;
 	ml_char_encoding_t  encoding ;
-	void * hold_handler;
 
 	if( magic != (u_int64_t) IM_API_COMPAT_CHECK_MAGIC)
 	{
@@ -1311,6 +1480,11 @@ im_new(
 
 	uim = NULL ;
 
+	if( getenv( "MOD_KEY_DEBUG"))
+	{
+		mod_key_debug = 1 ;
+	}
+
 #if 1
 #define  RESTORE_LOCALE
 #endif
@@ -1344,7 +1518,7 @@ im_new(
 		kik_locale_init( cur_locale) ;
 	#endif
 
-		mlterm_syms = export_syms ;
+		syms = export_syms ;
 
 		kik_list_new( im_uim_t , uim_list) ;
 
@@ -1354,24 +1528,14 @@ im_new(
 	/*
 	 * create I/O chanel for uim_helper_server
 	 */
-	if( helper_fd == -1 &&
-	    mlterm_syms &&
-	    mlterm_syms->x_term_manager_add_fd &&
-	    mlterm_syms->x_term_manager_remove_fd)
+	if( helper_fd == -1 && syms && syms->x_term_manager_add_fd &&
+	    syms->x_term_manager_remove_fd)
 	{
-		hold_handler = signal(SIGCHLD, SIG_DFL) ;
-
 		helper_fd = uim_helper_init_client_fd( helper_disconnected) ;
 
-		/* restoring */
-		signal(SIGCHLD, hold_handler) ;
-
-		(*mlterm_syms->x_term_manager_add_fd)(helper_fd ,
-						      helper_read_handler) ;
+		(*syms->x_term_manager_add_fd)( helper_fd , helper_read_handler) ;
 	}
 
-	hold_handler = NULL ;
-
 	if( engine == NULL || strlen( engine) == 0)
 	{
 	#ifdef  UIM_0_4_4_OR_LATER
@@ -1383,13 +1547,11 @@ im_new(
 
 	if( ! find_engine( engine , &encoding_name))
 	{
-		kik_error_printf( " Could not find '%s' conversion engine.\n" ,
-				  engine) ;
-
+		kik_error_printf( "%s: No such conversion engine.\n" , engine) ;
 		goto  error ;
 	}
 
-	if( (encoding = (*mlterm_syms->ml_get_char_encoding)( encoding_name)) == ML_UNKNOWN_ENCODING)
+	if( (encoding = (*syms->ml_get_char_encoding)( encoding_name)) == ML_UNKNOWN_ENCODING)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " %s is unknown encoding.\n" , encoding_name) ;
@@ -1408,35 +1570,31 @@ im_new(
 
 	uim->term_encoding = term_encoding ;
 	uim->encoding_name = encoding_name ;
-	uim->engine_name = strdup( engine) ;
 	uim->parser_uim = NULL ;
 	uim->parser_term = NULL ;
 	uim->conv = NULL ;
+	uim->pressing_mod_key = 0 ;
+	uim->mod_ignore_mask =  mod_ignore_mask ;
+	uim->cand_limit = 0 ;
 
 	if( uim->term_encoding != encoding)
 	{
-		if( ! ( uim->parser_uim = (*mlterm_syms->ml_parser_new)( encoding)))
+		if( ! ( uim->parser_uim = (*syms->ml_parser_new)( encoding)))
 		{
 			goto  error ;
 		}
 
-		if( ! ( uim->conv = (*mlterm_syms->ml_conv_new)( term_encoding)))
+		if( ! ( uim->conv = (*syms->ml_conv_new)( term_encoding)))
 		{
 			goto  error ;
 		}
 	}
 
-	if( ! ( uim->parser_term = (*mlterm_syms->ml_parser_new)( term_encoding)))
+	if( ! ( uim->parser_term = (*syms->ml_parser_new)( term_encoding)))
 	{
 		goto  error ;
 	}
 
-	/* XXX */
-	if( strcmp( engine , "prime") == 0 && ref_count == 0)
-	{
-		hold_handler = signal(SIGCHLD, SIG_DFL) ;
-	}
-
 	if( ! ( uim->context = uim_create_context( uim ,
 						   encoding_name ,
 						   NULL ,
@@ -1451,12 +1609,6 @@ im_new(
 		goto  error ;
 	}
 
-	/* restoring */
-	if( hold_handler)
-	{
-		signal(SIGCHLD, hold_handler) ;
-	}
-
 	uim_set_preedit_cb( uim->context ,
 			    preedit_clear ,
 			    preedit_pushback ,
@@ -1471,9 +1623,7 @@ im_new(
 	uim_set_prop_list_update_cb( uim->context , prop_list_update) ;
 	uim_set_prop_label_update_cb( uim->context , prop_label_update) ;
 
-#ifdef  IM_UIM_COMPAT_0_3_8
-	last_focused_uim = uim ;
-#endif
+	focused_uim = uim ;
 	uim_prop_list_update( uim->context) ;
 
 	/*
@@ -1496,17 +1646,11 @@ im_new(
 	return  (x_im_t*) uim ;
 
 error:
-	/* restoring */
-	if( hold_handler)
-	{
-		signal(SIGCHLD, hold_handler) ;
-	}
-
 	if( helper_fd != -1)
 	{
 		uim_helper_close_client_fd( helper_fd) ;
 
-		(*mlterm_syms->x_term_manager_remove_fd)( helper_fd) ;
+		(*syms->x_term_manager_remove_fd)( helper_fd) ;
 
 		helper_fd = -1 ;
 	}
@@ -1535,11 +1679,6 @@ error:
 			(*uim->conv->delete)( uim->conv) ;
 		}
 
-		if( uim->engine_name)
-		{
-			free( uim->engine_name) ;
-		}
-
 		free( uim) ;
 	}
 
@@ -1550,12 +1689,12 @@ error:
 /* --- API for external tools --- */
 
 im_info_t *
-im_get_info(
+im_uim_get_info(
 	char *  locale ,
 	char *  encoding
 	)
 {
-	im_info_t *  result ;
+	im_info_t *  result = NULL ;
 	uim_context  u ;
 	int  i ;
 
@@ -1582,12 +1721,12 @@ im_get_info(
 	result->id = strdup( "uim") ;
 	result->name = strdup( "uim") ;
 	result->num_of_args = uim_get_nr_im( u) + 1;
-	if( ! ( result->args = malloc( sizeof(char*) * result->num_of_args)))
+	if( ! ( result->args = calloc( result->num_of_args , sizeof( char*))))
 	{
 		goto  error ;
 	}
 
-	if( ! ( result->readable_args = malloc( sizeof(char*) * result->num_of_args)))
+	if( ! ( result->readable_args = calloc( result->num_of_args , sizeof( char*))))
 	{
 		free( result->args) ;
 		goto  error ;
@@ -1641,6 +1780,16 @@ error:
 
 	if( result)
 	{
+		if( result->args)
+		{
+			free( result->args) ;
+		}
+
+		if( result->readable_args)
+		{
+			free( result->readable_args) ;
+		}
+
 		free( result) ;
 	}
 
