@@ -1,5 +1,5 @@
 /*
- *	$Id: kik_dlfcn_dld.c,v 1.3 2004/10/06 05:26:14 seiichi Exp $
+ *	$Id: kik_dlfcn_dld.c,v 1.4 2005/02/18 14:32:00 seiichi Exp $
  */
 
 #include  "kik_dlfcn.h"
@@ -20,15 +20,31 @@ kik_dl_open(
 	)
 {
 	char *  path ;
+	shl_t  handle ;
 
 	if( ( path = alloca( strlen( dirpath) + strlen( name) + 7)) == NULL)
 	{
 		return  NULL ;
 	}
 
-	sprintf( path , "%slib%s.sl" , dirpath , name) ;
 
-	return  (kik_dl_handle_t)shl_load( path , BIND_DEFERRED , 0x0) ;
+	/*
+	 * libfoo.sl --> foo.sl
+	 */
+
+	sprintf( path , "%slib%s.sl" , dirpath , name) ;
+	if( ( handle = shl_load( path , BIND_DEFERRED , 0x0)))
+	{
+		return  (kik_dl_handle_t) handle ;
+	}
+
+	sprintf( path , "%s%s.sl" , dirpath , name) ;
+	if( ( handle = shl_load( path , BIND_DEFERRED , 0x0)))
+	{
+		return  (kik_dl_handle_t) handle ;
+	}
+
+	return  NULL ;
 }
 
 int
