@@ -1,5 +1,5 @@
 /*
- *	$Id: kik_dlfcn_ltdl.c,v 1.3 2004/10/06 05:26:14 seiichi Exp $
+ *	$Id: kik_dlfcn_ltdl.c,v 1.5 2005/02/18 14:32:00 seiichi Exp $
  */
 
 #include  "kik_dlfcn.h"
@@ -39,11 +39,23 @@ kik_dl_open(
 		return  NULL ;
 	}
 
+	/*
+	 * libfoo -> foo
+	 */
+
 	sprintf( path , "%slib%s" , dirpath , name) ;
+	if( ( handle = lt_dlopenext( path )))
+	{
+		return  (kik_dl_handle_t)handle;
+	}
 
-	handle = lt_dlopenext( path ) ;
+	sprintf( path , "%s%s" , dirpath , name) ;
+	if( ( handle = lt_dlopenext( path )))
+	{
+		return  (kik_dl_handle_t)handle;
+	}
 
-	return  (kik_dl_handle_t)handle;
+	return  NULL ;
 }
 
 int
@@ -81,6 +93,16 @@ kik_dl_is_module(
 		return  0 ;
 	}
 
-	return  1 ;
+	if( ( len = strlen( name)) < 3)
+	{
+		return  0 ;
+	}
+
+	if( strcmp( &name[len - 3] , ".la") == 0)
+	{
+		return  1 ;
+	}
+
+	return  0 ;
 }
 
