@@ -1,5 +1,5 @@
 /*
- *	$Id: mkf_char.h,v 1.4 2002/11/16 16:54:41 arakiken Exp $
+ *	$Id: mkf_char.h,v 1.5 2005/02/04 17:51:20 seiichi Exp $
  */
 
 #ifndef  __MKF_CHAR_H__
@@ -37,11 +37,11 @@ typedef struct  mkf_char
 } mkf_char_t ;
 
 
-inline u_int32_t  mkf_char_to_int( mkf_char_t *  ch) ;
+u_int32_t  mkf_char_to_int( mkf_char_t *  ch) ;
 
-inline u_char *  mkf_int_to_bytes( u_char *  bytes , size_t  len , u_int32_t  int_ch) ;
+u_char *  mkf_int_to_bytes( u_char *  bytes , size_t  len , u_int32_t  int_ch) ;
 
-inline u_int32_t  mkf_bytes_to_int( u_char *  bytes , size_t  len) ;
+u_int32_t  mkf_bytes_to_int( u_char *  bytes , size_t  len) ;
 
 
 #endif
