@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_color.h,v 1.3 2002/05/30 12:51:53 arakiken Exp $
+ *	$Id: ml_color.h,v 1.4 2004/12/07 03:37:00 h_minami Exp $
  */
 
 #ifndef  __ML_COLOR_H__
@@ -27,7 +27,7 @@ typedef enum  ml_color
 	ML_BOLD_COLOR_MASK = 0x8 ,
 	
 	ML_FG_COLOR = 0x10 ,
-	ML_BG_COLOR = 0x11 ,
+	ML_BG_COLOR = 0x11
 
 } ml_color_t ;
 
