@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_line.h,v 1.14 2004/03/04 15:59:02 arakiken Exp $
+ *	$Id: ml_line.h,v 1.15 2004/12/13 09:31:54 h_minami Exp $
  */
 
 #ifndef  __ML_LINE_H__
@@ -15,7 +15,7 @@ enum
 {
 	WRAPAROUND     = 0x01 ,
 	BREAK_BOUNDARY = 0x02 ,
-	SCROLL         = 0x04 ,
+	SCROLL         = 0x04
 } ;
 
 
