@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_model.h,v 1.8 2003/05/20 12:30:17 arakiken Exp $
+ *	$Id: ml_model.h,v 1.10 2004/10/09 04:39:47 arakiken Exp $
  */
 
 #ifndef  __ML_MODEL_H__
@@ -8,24 +8,10 @@
 
 #include  <kiklib/kik_types.h>
 
-#include  "ml_char.h"
+#include  "ml_str.h"
 #include  "ml_line.h"
 
 
-typedef struct  ml_cursor
-{
-	int  row ;
-	int  char_index ;
-	int  col ;
-	int  col_in_char ;
-	
-	int  saved_row ;
-	int  saved_char_index ;
-	int  saved_col ;
-	int8_t  is_saved ;
-	
-} ml_cursor_t ;
-
 typedef struct  ml_model
 {
 	/* private */
