@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_str.h,v 1.3 2004/02/22 08:19:10 arakiken Exp $
+ *	$Id: ml_str.h,v 1.4 2005/02/04 17:51:24 seiichi Exp $
  */
 
 #ifndef  __ML_STR_H__
@@ -21,13 +21,13 @@ int  ml_str_final( ml_char_t *  str , u_int  size) ;
 
 int  ml_str_delete( ml_char_t *  str , u_int  size) ;
 
-inline int  ml_str_copy( ml_char_t *  dst , ml_char_t *  src , u_int  size) ; 
+int  ml_str_copy( ml_char_t *  dst , ml_char_t *  src , u_int  size) ; 
 
-inline u_int  ml_str_cols( ml_char_t *  chars , u_int  len) ;
+u_int  ml_str_cols( ml_char_t *  chars , u_int  len) ;
 
-inline int  ml_str_equal( ml_char_t *  str1 , ml_char_t *  str2 , u_int  len) ;
+int  ml_str_equal( ml_char_t *  str1 , ml_char_t *  str2 , u_int  len) ;
 
-inline int  ml_str_bytes_equal( ml_char_t *  str1 , ml_char_t *  str2 , u_int  len) ;
+int  ml_str_bytes_equal( ml_char_t *  str1 , ml_char_t *  str2 , u_int  len) ;
 
 #ifdef  DEBUG
 
