@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_term.c,v 1.29 2004/05/16 06:21:02 seiichi Exp $
+ *	$Id: ml_term.c,v 1.32 2005/03/03 12:00:32 mfabian Exp $
  */
 
 #include  "ml_term.h"
@@ -88,6 +88,7 @@ ml_term_new(
 
 	term->win_name = NULL ;
 	term->icon_name = NULL ;
+	term->icon_path = NULL ;
 	term->is_mouse_pos_sending = 0 ;
 	term->is_app_keypad = 0 ;
 	term->is_app_cursor_keys = 0 ;
@@ -1148,6 +1149,18 @@ ml_term_set_icon_name(
 	return  1 ;
 }
 
+int
+ml_term_set_icon_path(
+	ml_term_t *  term ,
+	char *  path
+	)
+{
+	free( term->icon_path) ;
+	term->icon_path = strdup( path) ;
+
+	return 1 ;
+}
+
 char *
 ml_term_window_name(
 	ml_term_t *  term
@@ -1164,6 +1177,14 @@ ml_term_icon_name(
 	return  term->icon_name ;
 }
 
+char *
+ml_term_icon_path(
+	ml_term_t *  term
+	)
+{
+	return term->icon_path ;
+}
+
 int
 ml_term_start_config_menu(
 	ml_term_t *  term ,
