@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_term.h,v 1.25 2004/05/16 06:21:02 seiichi Exp $
+ *	$Id: ml_term.h,v 1.26 2005/02/17 01:59:57 h_minami Exp $
  */
 
 /*
@@ -53,7 +53,7 @@ typedef struct ml_term
 	 */
 	char *  win_name ;
 	char *  icon_name ;
-
+	char *  icon_path ;
 	int8_t  is_auto_encoding ;
 	
 	int8_t  is_mouse_pos_sending ;
@@ -227,10 +227,14 @@ int  ml_term_set_window_name( ml_term_t *  term , char *  name) ;
 
 int  ml_term_set_icon_name( ml_term_t *  term , char *  name) ;
 
+int  ml_term_set_icon_path( ml_term_t *  term , char *  path) ;
+
 char *  ml_term_window_name( ml_term_t *  term) ;
 
 char *  ml_term_icon_name( ml_term_t *  term) ;
 
+char *  ml_term_icon_path( ml_term_t *  term) ;
+
 int  ml_term_start_config_menu( ml_term_t *  term , char *  cmd_path , int  x , int  y , char *  display) ;
 
 
