@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_vt100_parser.c,v 1.40 2004/09/19 13:25:45 seiichi Exp $
+ *	$Id: ml_vt100_parser.c,v 1.45 2005/02/23 07:47:01 h_minami Exp $
  */
 
 #include  "ml_vt100_parser.h"
@@ -66,7 +66,7 @@
 #endif
 
 /* Optimization cooperating with OPTIMIZE_REDRAWING macro defined in ml_line.c. */
-#if  1
+#if  0
 #define  IGNORE_SPACE_FG_COLOR
 #endif
 
@@ -1211,6 +1211,7 @@ parse_vt100_escape_sequence(
 					#endif
 						else if( ps[0] == 3)
 						{
+							clear_display_all( vt100_parser) ; /* XTERM compatibility [#1048321] */
 							if( HAS_XTERM_LISTENER(vt100_parser,resize_columns))
 							{
 								stop_vt100_cmd( vt100_parser) ;
@@ -1325,18 +1326,29 @@ parse_vt100_escape_sequence(
 							/* scroll to bottom on key press */
 						}
 					#endif
-					#if  0
 						else if( ps[0] == 1047)
 						{
-							/* secondary screen w/ clearing */
+							// if( !titeInhibit)
+							save_cursor( vt100_parser) ;
+							ml_screen_use_alternative_edit(
+								vt100_parser->screen) ;
+							//
 						}
-					#endif
-					#if  0
 						else if( ps[0] == 1048)
 						{
-							/* alternative cursor save */
+							// if( !titeInhibit)
+							save_cursor( vt100_parser) ;
+							//
+						}
+						else if( ps[0] == 1049)
+						{
+							// if( !titeInhibit)
+							save_cursor( vt100_parser) ;
+							ml_screen_use_alternative_edit(
+								vt100_parser->screen) ;
+							clear_display_all( vt100_parser) ;
+							//
 						}
-					#endif
 						else
 						{
 						#ifdef  DEBUG
@@ -1368,6 +1380,7 @@ parse_vt100_escape_sequence(
 					#endif
 						else if( ps[0] == 3)
 						{
+							clear_display_all( vt100_parser) ; /* XTERM compatibility [#1048321] */
 							if( HAS_XTERM_LISTENER(vt100_parser,resize_columns))
 							{
 								stop_vt100_cmd( vt100_parser) ;
@@ -1481,18 +1494,29 @@ parse_vt100_escape_sequence(
 							/* scroll to bottom on key press */
 						}
 					#endif
-					#if  0
 						else if( ps[0] == 1047)
 						{
-							/* secondary screen w/ clearing */
+							// if( !titeInhibit)
+							clear_display_all( vt100_parser) ;
+							ml_screen_use_normal_edit(
+								vt100_parser->screen) ;
+							restore_cursor( vt100_parser) ;
+							//
 						}
-					#endif
-					#if  0
 						else if( ps[0] == 1048)
 						{
-							/* alternative cursor save */
+							// if( !titeInhibit)
+							restore_cursor( vt100_parser) ;
+							//
+						}
+						else if( ps[0] == 1049)
+						{
+							// if( !titeInhibit)
+							ml_screen_use_normal_edit(
+								vt100_parser->screen) ;
+							restore_cursor( vt100_parser) ;
+							//
 						}
-					#endif
 						else
 						{
 						#ifdef  DEBUG
