@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_color.c,v 1.15 2003/03/25 12:30:48 kubota Exp $
+ *	$Id: mc_color.c,v 1.18 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_color.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
 #include  <glib.h>
 #include  <c_intl.h>
@@ -72,7 +72,7 @@ static void init_i18ncolors(void)
 	return;
 }
 
-static char *untranslate(char *color)
+static const char *untranslate(const char *color)
 {
 	int j;
 
@@ -92,7 +92,7 @@ static char *translate(char *color)
 	return color;
 }
 
-static char *color_strncpy(char *dst, char *src)
+static char *color_strncpy(char *dst, const char *src)
 {
 	strncpy(dst, src, MC_COLOR_LEN);
 	dst[MC_COLOR_LEN-1] = 0;
