@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_flags.c,v 1.4 2003/03/02 08:16:28 kubota Exp $
+ *	$Id: mc_flags.c,v 1.6 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_flags.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <glib.h>
 #include  <c_intl.h>
 
