@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_gamma.c,v 1.6 2003/03/01 14:59:48 arakiken Exp $
+ *	$Id: mc_gamma.c,v 1.9 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_gamma.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
 #include  <glib.h>
 #include  <c_intl.h>
@@ -21,8 +21,8 @@
 
 /* --- static variables --- */
 
-static char *  new_gamma ;
-static char *  old_gamma ;
+static char *  new_gamma = NULL;
+static char *  old_gamma = NULL;
 static int is_changed;
 
 
@@ -34,7 +34,8 @@ gamma_selected(
 	gpointer  data
 	)
 {
-	new_gamma = gtk_entry_get_text(GTK_ENTRY(widget)) ;
+	free( new_gamma);
+	new_gamma = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1) ;
 	
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " %s gamma is selected.\n" , new_gamma) ;
@@ -63,7 +64,7 @@ mc_gamma_config_widget_new(void)
 		"10" ,
 	} ;
 
-	new_gamma = old_gamma = mc_get_str_value( "gamma") ;
+	new_gamma = strdup( old_gamma = mc_get_str_value( "gamma")) ;
 	is_changed = 0;
 
 	return  mc_combo_new_with_width(_("Gamma"), gammas,
