@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_iscii_lang.c,v 1.8 2003/03/01 14:59:48 arakiken Exp $
+ *	$Id: mc_iscii_lang.c,v 1.11 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_iscii_lang.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
 #include  <glib.h>
 #include  <c_intl.h>
@@ -21,8 +21,8 @@
 
 /* --- static variables --- */
 
-static char *  new_iscii_lang ;
-static char *  old_iscii_lang ;
+static char *  new_iscii_lang = NULL;
+static char *  old_iscii_lang = NULL;
 static int is_changed;
 
 
@@ -34,7 +34,8 @@ iscii_lang_selected(
 	gpointer  data
 	)
 {
-	new_iscii_lang = gtk_entry_get_text(GTK_ENTRY(widget)) ;
+	free( new_iscii_lang);
+	new_iscii_lang = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1) ;
 
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " %s iscii_lang is selected.\n" , new_iscii_lang) ;
@@ -65,7 +66,7 @@ mc_iscii_lang_config_widget_new(void)
 
 	} ;
 	
-	new_iscii_lang = old_iscii_lang = mc_get_str_value( "iscii_lang") ;
+	new_iscii_lang = strdup( old_iscii_lang = mc_get_str_value( "iscii_lang")) ;
 	is_changed = 0;
 
 	return  mc_combo_new( _("ISCII language") , iscii_langs ,
