@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_logsize.c,v 1.11 2003/03/01 14:59:48 arakiken Exp $
+ *	$Id: mc_logsize.c,v 1.14 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_logsize.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
 #include  <glib.h>
 #include  <c_intl.h>
@@ -21,8 +21,8 @@
 
 /* --- static variables --- */
 
-static char *  new_logsize ;
-static char *  old_logsize ;
+static char *  new_logsize = NULL;
+static char *  old_logsize = NULL;
 static int is_changed;
 
 
@@ -34,7 +34,8 @@ logsize_selected(
 	gpointer  data
 	)
 {
-	new_logsize = gtk_entry_get_text(GTK_ENTRY(widget)) ;
+	free( new_logsize);
+	new_logsize = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1) ;
 	
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " %s logsize is selected.\n" , new_logsize) ;
@@ -57,7 +58,7 @@ mc_logsize_config_widget_new(void)
 		"1024" ,
 	} ;
 
-	new_logsize = old_logsize = mc_get_str_value( "logsize") ;
+	new_logsize = strdup( old_logsize = mc_get_str_value( "logsize")) ;
 	is_changed = 0;
 
 	return  mc_combo_new_with_width(_("Backlog size (lines)"), logsizes,
