@@ -1,5 +1,5 @@
 /*
- *	$Id: x_color.c,v 1.6 2003/12/27 09:06:22 arakiken Exp $
+ *	$Id: x_color.c,v 1.7 2004/12/13 09:31:55 h_minami Exp $
  */
 
 #include  "x_color.h"
@@ -123,9 +123,9 @@ is_rgb_color_name(
 {
 	if( *name == '#')
 	{
-		int  _red ;
-		int  _green ;
-		int  _blue ;
+		u_int  _red ;
+		u_int  _green ;
+		u_int  _blue ;
 		
 		if( strlen( name + 1) == 6 &&
 			sscanf( name + 1 , "%2x%2x%2x" , &_red , &_green , &_blue) == 3)
