@@ -1,20 +1,20 @@
 /*
- *	$Id: x_color.h,v 1.2 2003/05/16 10:08:38 arakiken Exp $
+ *	$Id: x_color.h,v 1.4 2003/12/27 09:06:22 arakiken Exp $
  */
 
 #ifndef  __X_COLOR_H__
 #define  __X_COLOR_H__
 
 
-/* this must be included before including Xft.h(before XFree86-4.0.x) */
+/* This must be included ahead of Xft.h on XFree86-4.0.x or before. */
 #include  <X11/Xlib.h>
 
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 #include  <X11/Xft/Xft.h>
 #endif
 
 
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 typedef XftColor  x_color_t ;
 #else
 typedef XColor  x_color_t ;
