@@ -1,5 +1,5 @@
 /*
- *	$Id: x_font.c,v 1.12 2003/09/14 05:23:08 arakiken Exp $
+ *	$Id: x_font.c,v 1.20 2004/03/22 12:21:55 h_minami Exp $
  */
 
 #include  "x_font.h"
@@ -18,7 +18,6 @@
 #define  FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p) \
 	for( (font_encoding_p) = &csinfo->encoding_names[0] ; *(font_encoding_p) ; (font_encoding_p) ++)
 
-
 #if  0
 #define  __DEBUG
 #endif
@@ -27,15 +26,15 @@
 typedef struct  cs_info
 {
 	mkf_charset_t  cs ;
-	
-	/* 
+
+	/*
 	 * default encoding.
 	 *
 	 * !! Notice !!
 	 * the last element must be NULL.
 	 */
 	char *   encoding_names[3] ;
-	
+
 } cs_info_t ;
 
 
@@ -59,7 +58,7 @@ static cs_info_t  cs_info_table[] =
 	{ ISO8859_13_R , { "iso8859-13" , NULL , NULL , } , } ,
 	{ ISO8859_14_R , { "iso8859-14" , NULL , NULL , } , } ,
 	{ ISO8859_15_R , { "iso8859-15" , NULL , NULL , } , } ,
-	
+
 	/*
 	 * XXX
 	 * The encoding of TCVN font is iso8859-1 , and its font family is
@@ -75,7 +74,7 @@ static cs_info_t  cs_info_table[] =
 	{ GEORGIAN_PS , { NULL , NULL , NULL , } , } ,
 	{ CP1251 , { NULL , NULL , NULL , } , } ,
 	{ CP1255 , { NULL , NULL , NULL , } , } ,
-	
+
 	{ JISX0201_KATA , { "jisx0201.1976-0" , NULL , NULL , } , } ,
 	{ JISX0201_ROMAN , { "jisx0201.1976-0" , NULL , NULL , } , } ,
 	{ JISC6226_1978 , { "jisx0208.1978-0" , "jisx0208.1983-0" , NULL , } , } ,
@@ -85,7 +84,7 @@ static cs_info_t  cs_info_table[] =
 	{ JISX0213_2000_1 , { "jisx0213.2000-1" , NULL , NULL , } , } ,
 	{ JISX0213_2000_2 , { "jisx0213.2000-2" , NULL , NULL , } , } ,
 	{ KSC5601_1987 , { "ksc5601.1987-0" , "ksx1001.1997-0" , NULL , } , } ,
-	
+
 	/*
 	 * XXX
 	 * UHC and JOHAB fonts are not used at the present time.
@@ -106,7 +105,7 @@ static cs_info_t  cs_info_table[] =
 	{ CNS11643_1992_6 , { "cns11643.1992-6" , "cns11643.1992.6-0" , NULL , } , } ,
 	{ CNS11643_1992_7 , { "cns11643.1992-7" , "cns11643.1992.7-0" , NULL , } , } ,
 	{ ISO10646_UCS4_1 , { "iso10646-1" , NULL , NULL , } , } ,
-	
+
 } ;
 
 static int  compose_dec_special_font ;
@@ -129,15 +128,53 @@ get_cs_info(
 			return  &cs_info_table[count] ;
 		}
 	}
-	
+
 #ifdef  DEBUG
-	kik_warn_printf( KIK_DEBUG_TAG " not supported cs(%x).\n" , cs) ; 
+	kik_warn_printf( KIK_DEBUG_TAG " not supported cs(%x).\n" , cs) ;
 #endif
 
 	return  NULL ;
 }
 
-#ifdef  ANTI_ALIAS
+static int
+set_decsp_font(
+	x_font_t *  font
+	)
+{
+	/*
+	 * freeing font->xfont or font->xft_font
+	 */
+#ifdef  USE_TYPE_XFT
+	if( font->xft_font)
+	{
+		XftFontClose( font->display , font->xft_font) ;
+		font->xft_font = NULL ;
+	}
+#endif
+#ifdef  USE_TYPE_XCORE
+	if( font->xfont)
+	{
+		XFreeFont( font->display , font->xfont) ;
+		font->xfont = NULL ;
+	}
+#endif
+
+	if( ( font->decsp_font = x_decsp_font_new( font->display , font->width , font->height ,
+					font->height_to_baseline)) == NULL)
+	{
+		return  0 ;
+	}
+
+	/* decsp_font is impossible to draw double with. */
+	font->is_double_drawing = 0 ;
+
+	/* decsp_font is always fixed pitch. */
+	font->is_proportional = 0 ;
+
+	return  1 ;
+}
+
+#ifdef  USE_TYPE_XFT
 
 static u_int
 xft_calculate_char_width(
@@ -182,8 +219,365 @@ xft_calculate_char_width(
 	return  extents.xOff ;
 }
 
+static int
+parse_xft_font_name(
+	char **  font_family ,
+	char **  font_encoding ,
+	char **  percent ,
+	char *  font_name
+	)
+{
+	/*
+	 * XftFont format.
+	 * [Font Family]-[Font Encoding](:[Percentage])
+	 */
+
+	if( ( *font_family = font_name))
+	{
+		/*
+		 * It seems that XftFont format allows hyphens to be escaped.
+		 * (e.g. Foo\-Bold-iso10646-1)
+		 */
+		int  i ;
+		int  j ;
+		char *  s ;
+
+		s = font_name ;
+		for( i = 0 , j = 0 ; s[i] && s[i] != '-' ; i ++ , j ++)
+		{
+			/* escaped? */
+			if( s[i] == '\\' && s[i + 1])
+			{
+				/* skip backslash */
+				i ++ ;
+			}
+			s[j] = s[i] ;
+		}
+		if( !s[i])
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " encoding part is missing(%s).\n" ,
+				font_name) ;
+		#endif
+			return  0 ;
+		}
+		/* replace delimiter */
+		s[j] = '\0' ;
+		/* move forward to the next token */
+		font_name = s + i + 1 ;
+	}
+
+	if( *font_family == NULL || font_name == NULL)
+	{
+	#ifdef  DEBUG
+		kik_warn_printf( KIK_DEBUG_TAG " illegal true type font name(%s).\n" ,
+			font_name) ;
+	#endif
+
+		return  0 ;
+	}
+
+	if( ( *font_encoding = kik_str_sep( &font_name , ":")) == NULL)
+	{
+	#ifdef  DEBUG
+		kik_warn_printf( KIK_DEBUG_TAG " illegal true type font name(%s).\n" ,
+			font_name) ;
+	#endif
+
+		return  0 ;
+	}
+
+	/* may be NULL */
+	*percent = font_name ;
+
+	return  1 ;
+}
+
+static u_int
+get_xft_col_width(
+	x_font_t *  font ,
+	char *  family ,
+	u_int  fontsize
+	)
+{
+	XftFont *  xfont ;
+
+	/*
+	 * XXX
+	 * DefaultScreen() should not be used , but ...
+	 */
+	if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+		XFT_FAMILY , XftTypeString , family ,
+		XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
+		XFT_ENCODING , XftTypeString , "iso8859-1" ,
+		XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL , NULL)))
+	{
+		u_int  w_width ;
+
+		w_width = xft_calculate_char_width( font->display , xfont , "W" , 1) ;
+
+		XftFontClose( font->display , xfont) ;
+
+		if( w_width > 0)
+		{
+			return  w_width ;
+		}
+	}
+
+	/* XXX this may be inaccurate. */
+	return  fontsize / 2 ;
+}
+
+static int
+set_xft_font(
+	x_font_t *  font ,
+	char *  fontname ,
+	u_int  fontsize ,
+	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set. */
+	int  use_medium_for_bold ,
+	int  is_aa
+	)
+{
+	int  weight ;
+	cs_info_t *  csinfo ;
+	u_int  ch_width ;
+	XftFont *  xfont ;
+	char **  font_encoding_p ;
+
+	if( ( csinfo = get_cs_info( FONT_CS(font->id))) == NULL)
+	{
+	#ifdef  DEBUG
+		kik_warn_printf( KIK_DEBUG_TAG " charset(0x%.2x) is not supported.\n" ,
+			FONT_CS(font->id)) ;
+	#endif
+
+		return  0 ;
+	}
+
+	if( font->id & FONT_BOLD)
+	{
+		weight = XFT_WEIGHT_BOLD ;
+	}
+	else
+	{
+		weight = XFT_WEIGHT_MEDIUM ;
+	}
+
+	if( fontname)
+	{
+		char *  p ;
+		char *  font_family ;
+		char *  font_encoding ;
+		char *  percent_str ;
+		u_int  percent ;
+
+		if( ( p = kik_str_alloca_dup( fontname)) == NULL)
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG " alloca() failed.\n") ;
+		#endif
+
+			return  0 ;
+		}
+
+		if( parse_xft_font_name( &font_family , &font_encoding , &percent_str , p))
+		{
+			if( col_width == 0)
+			{
+				/* basic font (e.g. usascii) width */
+
+				if( percent_str == NULL || ! kik_str_to_uint( &percent , percent_str) ||
+					percent == 0)
+				{
+					ch_width = get_xft_col_width( font , font_family , fontsize) ;
+				}
+				else
+				{
+					ch_width = (fontsize * font->cols * percent) / 200 ;
+				}
+
+				if( font->is_vertical)
+				{
+					/*
+					 * !! Notice !!
+					 * The width of full and half character font is the same.
+					 */
+					ch_width *= 2 ;
+				}
+			}
+			else
+			{
+				if( font->is_vertical)
+				{
+					/*
+					 * !! Notice !!
+					 * The width of full and half character font is the same.
+					 */
+					ch_width = col_width ;
+				}
+				else
+				{
+					ch_width = col_width * font->cols ;
+				}
+			}
+
+			if( font->is_var_col_width)
+			{
+				if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+						XFT_FAMILY , XftTypeString , font_family ,
+						XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
+						XFT_ENCODING , XftTypeString , font_encoding ,
+						XFT_WEIGHT , XftTypeInteger , weight ,
+						XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL ,
+						XFT_ANTIALIAS , XftTypeBool , is_aa ? True : False ,
+						NULL)))
+				{
+					goto  font_found ;
+				}
+			}
+			else
+			{
+				if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+						XFT_FAMILY , XftTypeString , font_family ,
+						XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
+						XFT_ENCODING , XftTypeString , font_encoding ,
+						XFT_CHAR_WIDTH , XftTypeInteger , ch_width ,
+						XFT_WEIGHT , XftTypeInteger , weight ,
+						XFT_SPACING , XftTypeInteger , XFT_CHARCELL ,
+						XFT_ANTIALIAS , XftTypeBool , is_aa ? True : False ,
+						NULL)))
+				{
+					goto  font_found ;
+				}
+			}
+		}
+
+		kik_msg_printf( " font %s (for size %d) couln't be loaded.\n" , fontname , fontsize) ;
+	}
+
+	if( col_width == 0)
+	{
+		/* basic font (e.g. usascii) width */
+		ch_width = get_xft_col_width( font , "" , fontsize) ;
+
+		/*
+		 * !! Notice !!
+		 * The width of full and half character font is the same.
+		 */
+		if( font->is_vertical)
+		{
+			ch_width *= 2 ;
+		}
+	}
+	else
+	{
+		if( font->is_vertical)
+		{
+			/*
+			 * !! Notice !!
+			 * The width of full and half character font is the same.
+			 */
+			ch_width = col_width ;
+		}
+		else
+		{
+			ch_width = col_width * font->cols ;
+		}
+	}
+
+	FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p)
+	{
+		if( font->is_var_col_width)
+		{
+			if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+					XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
+					XFT_ENCODING , XftTypeString , *font_encoding_p ,
+					XFT_WEIGHT , XftTypeInteger , weight ,
+					XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL ,
+					XFT_ANTIALIAS , XftTypeBool , is_aa ? True : False ,
+					NULL)))
+			{
+				goto  font_found ;
+			}
+		}
+		else
+		{
+			if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+					XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
+					XFT_ENCODING , XftTypeString , *font_encoding_p ,
+					XFT_WEIGHT , XftTypeInteger , weight ,
+					XFT_CHAR_WIDTH , XftTypeInteger , ch_width ,
+					XFT_SPACING , XftTypeInteger , XFT_CHARCELL ,
+					XFT_ANTIALIAS , XftTypeBool , is_aa ? True : False ,
+					NULL)))
+			{
+				goto  font_found ;
+			}
+		}
+	}
+
+#ifdef  DEBUG
+	kik_warn_printf( KIK_DEBUG_TAG " XftFontOpen(%s) failed.\n" , fontname) ;
 #endif
 
+	return  0 ;
+
+font_found:
+
+	if( weight == XFT_WEIGHT_BOLD &&
+		XftPatternGetInteger( xfont->pattern , XFT_WEIGHT , 0 , &weight) == XftResultMatch &&
+		weight != XFT_WEIGHT_BOLD)
+	{
+		font->is_double_drawing = 1 ;
+	}
+	else
+	{
+		font->is_double_drawing = 0 ;
+	}
+
+	font->xft_font = xfont ;
+
+	font->height = xfont->height ;
+	font->height_to_baseline = xfont->ascent ;
+
+	font->x_off = 0 ;
+	font->width = ch_width ;
+
+	font->is_proportional = font->is_var_col_width ;
+
+	/*
+	 * checking if font height/height_to_baseline member is sane.
+	 * font width must be always sane.
+	 */
+
+	if( font->height == 0)
+	{
+		/* XXX this may be inaccurate. */
+		font->height = fontsize ;
+	}
+
+	if( font->height_to_baseline == 0)
+	{
+		/* XXX this may be inaccurate. */
+		font->height_to_baseline = fontsize ;
+	}
+
+	/*
+	 * set_decsp_font() is called after dummy font is loaded to get font metrics.
+	 * Since dummy font encoding is "iso8859-1", loading rarely fails.
+	 */
+	/* XXX dec specials must always be composed for now */
+	if( /* compose_dec_special_font && */ FONT_CS(font->id) == DEC_SPECIAL)
+	{
+		return  set_decsp_font( font) ;
+	}
+
+	return  1 ;
+}
+
+#endif  /* USE_TYPE_XFT */
+
+#ifdef  USE_TYPE_XCORE
 
 static u_int
 calculate_char_width(
@@ -209,11 +603,11 @@ calculate_char_width(
 	else if( len == 4)
 	{
 		/* is UCS4 */
-		
+
 		XChar2b  c ;
 
 		/* dealing as UCS2 */
-		
+
 		c.byte1 = ch[2] ;
 		c.byte2 = ch[3] ;
 
@@ -257,111 +651,6 @@ parse_xfont_name(
 	return  1 ;
 }
 
-#ifdef  ANTI_ALIAS
-
-static int
-parse_xft_font_name(
-	char **  font_family ,
-	char **  font_encoding ,
-	char **  percent ,
-	char *  font_name
-	)
-{
-	/*
-	 * XftFont format.
-	 * [Font Family]-[Font Encoding](:[Percentage])
-	 */
-
-	if( ( *font_family = font_name))
-	{
-		/*
-		 * It seems that XftFont format allows hyphens to be escaped.
-		 * (e.g. Foo\-Bold-iso10646-1)
-		 */
-		int  i ;
-		int  j ;
-		char *  s ;
-
-		s = font_name ;
-		for( i = 0 , j = 0 ; s[i] && s[i] != '-' ; i ++ , j ++)
-		{
-			/* escaped? */
-			if( s[i] == '\\' && s[i + 1])
-			{
-				/* skip backslash */
-				i ++ ;
-			}
-			s[j] = s[i] ;
-		}
-		/* replace delimiter */
-		s[j] = '\0' ;
-		/* move forward to the next token */
-		font_name = s + i + 1 ;
-	}
-
-	if( *font_family == NULL || font_name == NULL)
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " illegal true type font name(%s).\n" ,
-			font_name) ;
-	#endif
-
-		return  0 ;
-	}
-
-	if( ( *font_encoding = kik_str_sep( &font_name , ":")) == NULL)
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " illegal true type font name(%s).\n" ,
-			font_name) ;
-	#endif
-
-		return  0 ;
-	}
-
-	/* may be NULL */
-	*percent = font_name ;
-
-	return  1 ;
-}
-
-static u_int
-get_xft_col_width(
-	x_font_t *  font ,
-	char *  family ,
-	u_int  fontsize
-	)
-{
-	XftFont *  xfont ;
-	
-	/*
-	 * XXX
-	 * DefaultScreen() should not be used , but ...
-	 */
-	if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
-		XFT_FAMILY , XftTypeString , family ,
-		XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
-		XFT_ENCODING , XftTypeString , "iso8859-1" ,
-		XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL , NULL)))
-	{
-		u_int  w_width ;
-		
-		w_width = xft_calculate_char_width( font->display , xfont , "W" , 1) ;
-		
-		XftFontClose( font->display , xfont) ;
-		
-		if( w_width > 0)
-		{
-			return  w_width ;
-		}
-	}
-	
-	/* XXX this may be inaccurate. */
-	return  fontsize / 2 ;
-}
-
-#endif
-
 static XFontStruct *
 load_xfont(
 	Display *  display ,
@@ -382,7 +671,7 @@ load_xfont(
 	max_len = 3 /* gnu */ + strlen(family) + 7 /* unifont */ + strlen( weight) +
 		strlen( slant) + strlen( width) + 2 /* lang */ + DIGIT_STR_LEN(fontsize) +
 		strlen( spacing) + strlen( encoding) + 20 ;
-	
+
 	if( ( fontname = alloca( max_len)) == NULL)
 	{
 		return  0 ;
@@ -403,7 +692,7 @@ load_xfont(
 	if( strcmp( encoding , "iso10646-1") == 0 && strcmp( family , "biwidth") == 0)
 	{
 		/* XFree86 Unicode font */
-		
+
 		kik_snprintf( fontname , max_len , "-*-*-%s-%s-%s-%s-%d-*-*-*-%s-*-%s" ,
 			weight , slant , width , kik_get_lang() , fontsize , spacing , encoding) ;
 
@@ -424,7 +713,7 @@ load_xfont(
 		#ifdef  __DEBUG
 			kik_debug_printf( KIK_DEBUG_TAG " loading %s.\n" , fontname) ;
 		#endif
-		
+
 			if( ( xfont = XLoadQueryFont( display , fontname)))
 			{
 				return  xfont ;
@@ -432,14 +721,14 @@ load_xfont(
 		}
 
 		/* GNU Unifont */
-		
+
 		kik_snprintf( fontname , max_len , "-gnu-unifont-%s-%s-%s--%d-*-*-*-%s-*-%s" ,
 			weight , slant , width , fontsize , spacing , encoding) ;
 
 	#ifdef  __DEBUG
 		kik_debug_printf( KIK_DEBUG_TAG " loading %s.\n" , fontname) ;
 	#endif
-		
+
 		return  XLoadQueryFont( display , fontname) ;
 	}
 	else
@@ -448,43 +737,6 @@ load_xfont(
 	}
 }
 
-static int
-set_decsp_font(
-	x_font_t *  font
-	)
-{
-	/*
-	 * freeing font->xfont or font->xft_font
-	 */
-#ifdef  ANTI_ALIAS
-	if( font->xft_font)
-	{
-		XftFontClose( font->display , font->xft_font) ;
-		font->xft_font = NULL ;
-	}
-	else
-#endif
-	if( font->xfont)
-	{
-		XFreeFont( font->display , font->xfont) ;
-		font->xfont = NULL ;
-	}
-	
-	if( ( font->decsp_font = x_decsp_font_new( font->display , font->width , font->height ,
-					font->height_to_baseline)) == NULL)
-	{
-		return  0 ;
-	}
-
-	/* decsp_font is impossible to draw double with. */
-	font->is_double_drawing = 0 ;
-
-	/* decsp_font is always fixed pitch. */
-	font->is_proportional = 0 ;
-
-	return  1 ;
-}
-
 static int
 set_xfont(
 	x_font_t *  font ,
@@ -509,7 +761,7 @@ set_xfont(
 		kik_warn_printf( KIK_DEBUG_TAG " get_cs_info(cs %x(id %x)) failed.\n" ,
 			FONT_CS(font->id) , font->id) ;
 	#endif
-	
+
 		return  0 ;
 	}
 
@@ -527,16 +779,16 @@ set_xfont(
 		char *  p ;
 		char *  font_xlfd ;
 		char *  percent_str ;
-		
+
 		if( ( p = kik_str_alloca_dup( fontname)) == NULL)
 		{
 		#ifdef  DEBUG
 			kik_warn_printf( KIK_DEBUG_TAG " alloca() failed.\n") ;
 		#endif
-		
+
 			return  0 ;
 		}
-		
+
 		if( parse_xfont_name( &font_xlfd , &percent_str , p))
 		{
 		#ifdef __DEBUG
@@ -552,18 +804,18 @@ set_xfont(
 
 				goto  font_found ;
 			}
-			
+
 			kik_msg_printf( " font %s couln't be loaded.\n" , font_xlfd) ;
 		}
 	}
-	
+
 	percent = 0 ;
-	
+
 	/*
 	 * searching apropriate font by using font info.
 	 */
 
-#ifdef  __DEBUG	 
+#ifdef  __DEBUG
 	kik_debug_printf( "font for id %x will be loaded.\n" , font->id) ;
 #endif
 
@@ -577,7 +829,7 @@ set_xfont(
 	}
 
 	slant = "r" ;
-	
+
 	width = "normal" ;
 
 	if( font->id & FONT_BIWIDTH)
@@ -597,7 +849,7 @@ set_xfont(
 			goto  font_found ;
 		}
 	}
-	
+
 	if( font->id & FONT_BOLD)
 	{
 		FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p)
@@ -608,7 +860,7 @@ set_xfont(
 				goto  font_found ;
 			}
 		}
-		
+
 		/*
 		 * loading variable width font :(
 		 */
@@ -621,7 +873,7 @@ set_xfont(
 				goto   font_found ;
 			}
 		}
-		
+
 		FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p)
 		{
 			if( ( xfont = load_xfont( font->display , family , weight , "*" , "*" ,
@@ -630,7 +882,7 @@ set_xfont(
 				goto   font_found ;
 			}
 		}
-		
+
 		/* no bold font is found. */
 		font->is_double_drawing = 1 ;
 	}
@@ -647,7 +899,7 @@ set_xfont(
 	/*
 	 * loading variable width font :(
 	 */
-	 
+
 	FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p)
 	{
 		if( ( xfont = load_xfont( font->display , family , "*" , "*" , "*" , fontsize ,
@@ -656,7 +908,7 @@ set_xfont(
 			goto   font_found ;
 		}
 	}
-	
+
 	FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p)
 	{
 		if( ( xfont = load_xfont( font->display , family , "*" , "*" , "*" , fontsize ,
@@ -671,7 +923,7 @@ set_xfont(
 font_found:
 
 	font->xfont = xfont ;
-	
+
 	font->height = xfont->ascent + xfont->descent ;
 	font->height_to_baseline = xfont->ascent ;
 
@@ -735,14 +987,14 @@ font_found:
 				" max font width(%d) and min one(%d) are mismatched.\n" ,
 				xfont->max_bounds.width , xfont->min_bounds.width) ;
 		#endif
-		
+
 			font->is_proportional = 1 ;
 		}
 		else
 		{
 			font->is_proportional = 0 ;
 		}
-		
+
 		font->width = xfont->max_bounds.width ;
 	}
 
@@ -751,7 +1003,7 @@ font_found:
 	if( col_width == 0)
 	{
 		/* standard(usascii) font */
-		
+
 		if( percent > 0)
 		{
 			u_int  ch_width ;
@@ -777,7 +1029,7 @@ font_found:
 				{
 					font->x_off = (ch_width - font->width) / 2 ;
 				}
-				
+
 				font->width = ch_width ;
 			}
 		}
@@ -808,16 +1060,16 @@ font_found:
 			 * !! Notice !!
 			 * The width of full and half character font is the same.
 			 */
-			 
+
 			if( font->width != col_width)
 			{
 				font->is_proportional = 1 ;
-				
+
 				if( font->width < col_width)
 				{
 					font->x_off = (col_width - font->width) / 2 ;
 				}
-				
+
 				font->width = col_width ;
 			}
 		}
@@ -832,12 +1084,12 @@ font_found:
 			#endif
 
 				font->is_proportional = 1 ;
-				
+
 				if( font->width < col_width * font->cols)
 				{
 					font->x_off = (col_width * font->cols - font->width) / 2 ;
 				}
-				
+
 				font->width = col_width * font->cols ;
 			}
 		}
@@ -847,7 +1099,7 @@ font_found:
 	/*
 	 * checking if font width/height/height_to_baseline member is sane.
 	 */
-	 
+
 	if( font->width == 0)
 	{
 	#ifdef  DEBUG
@@ -855,7 +1107,7 @@ font_found:
 	#endif
 
 		font->is_proportional = 1 ;
-		
+
 		/* XXX this may be inaccurate. */
 		font->width = (fontsize / 2) * font->cols ;
 	}
@@ -872,245 +1124,6 @@ font_found:
 		font->height_to_baseline = fontsize ;
 	}
 
-	/*
-	 * set_decsp_font() is called after dummy font is loaded to get font metrics.
-	 * Since dummy font encoding is "iso8859-1", loading rarely fails.
-	 */
-	if( compose_dec_special_font && FONT_CS(font->id) == DEC_SPECIAL)
-	{
-		return  set_decsp_font( font) ;
-	}
-	
-	return  1 ;
-}
-
-#ifdef  ANTI_ALIAS
-
-static int
-set_xft_font(
-	x_font_t *  font ,
-	char *  fontname ,
-	u_int  fontsize ,
-	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set. */
-	int  use_medium_for_bold
-	)
-{
-	int  weight ;
-	cs_info_t *  csinfo ;
-	u_int  ch_width ;
-	XftFont *  xfont ;
-	char **  font_encoding_p ;
-
-	if( ( csinfo = get_cs_info( FONT_CS(font->id))) == NULL)
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " charset(0x%.2x) is not supported.\n" ,
-			FONT_CS(font->id)) ;
-	#endif
-		
-		return  0 ;
-	}
-
-	if( font->id & FONT_BOLD)
-	{
-		weight = XFT_WEIGHT_BOLD ;
-	}
-	else
-	{
-		weight = XFT_WEIGHT_MEDIUM ;
-	}
-	
-	if( fontname)
-	{
-		char *  p ;
-		char *  font_family ;
-		char *  font_encoding ;
-		char *  percent_str ;
-		u_int  percent ;
-
-		if( ( p = kik_str_alloca_dup( fontname)) == NULL)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " alloca() failed.\n") ;
-		#endif
-		
-			return  0 ;
-		}
-
-		if( parse_xft_font_name( &font_family , &font_encoding , &percent_str , p))
-		{
-			if( col_width == 0)
-			{
-				/* basic font (e.g. usascii) width */
-				
-				if( percent_str == NULL || ! kik_str_to_uint( &percent , percent_str) ||
-					percent == 0)
-				{
-					ch_width = get_xft_col_width( font , font_family , fontsize) ;
-				}
-				else
-				{
-					ch_width = (fontsize * font->cols * percent) / 200 ;
-				}
-				
-				if( font->is_vertical)
-				{
-					/*
-					 * !! Notice !!
-					 * The width of full and half character font is the same.
-					 */
-					ch_width *= 2 ;
-				}
-			}
-			else
-			{
-				if( font->is_vertical)
-				{
-					/*
-					 * !! Notice !!
-					 * The width of full and half character font is the same.
-					 */
-					ch_width = col_width ;
-				}
-				else
-				{
-					ch_width = col_width * font->cols ;
-				}
-			}
-
-			if( font->is_var_col_width)
-			{
-				if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
-						XFT_FAMILY , XftTypeString , font_family ,
-						XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
-						XFT_ENCODING , XftTypeString , font_encoding ,
-						XFT_WEIGHT , XftTypeInteger , weight ,
-						XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL , NULL)))
-				{
-					goto  font_found ;
-				}
-			}
-			else
-			{
-				if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
-						XFT_FAMILY , XftTypeString , font_family ,
-						XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
-						XFT_ENCODING , XftTypeString , font_encoding ,
-						XFT_CHAR_WIDTH , XftTypeInteger , ch_width ,
-						XFT_WEIGHT , XftTypeInteger , weight ,
-						XFT_SPACING , XftTypeInteger , XFT_CHARCELL , NULL)))
-				{
-					goto  font_found ;
-				}
-			}
-		}
-		
-		kik_msg_printf( " font %s (for size %d) couln't be loaded.\n" , fontname , fontsize) ;
-	}
-	
-	if( col_width == 0)
-	{
-		/* basic font (e.g. usascii) width */
-		ch_width = get_xft_col_width( font , "" , fontsize) ;
-
-		/*
-		 * !! Notice !!
-		 * The width of full and half character font is the same.
-		 */
-		if( font->is_vertical)
-		{
-			ch_width *= 2 ;
-		}
-	}
-	else
-	{
-		if( font->is_vertical)
-		{
-			/*
-			 * !! Notice !!
-			 * The width of full and half character font is the same.
-			 */
-			ch_width = col_width ;
-		}
-		else
-		{
-			ch_width = col_width * font->cols ;
-		}
-	}
-
-	FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p)
-	{
-		if( font->is_var_col_width)
-		{
-			if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
-					XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
-					XFT_ENCODING , XftTypeString , *font_encoding_p ,
-					XFT_WEIGHT , XftTypeInteger , weight ,
-					XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL , NULL)))
-			{
-				goto  font_found ;
-			}
-		}
-		else
-		{
-			if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
-					XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
-					XFT_ENCODING , XftTypeString , *font_encoding_p ,
-					XFT_WEIGHT , XftTypeInteger , weight ,
-					XFT_CHAR_WIDTH , XftTypeInteger , ch_width ,
-					XFT_SPACING , XftTypeInteger , XFT_CHARCELL , NULL)))
-			{
-				goto  font_found ;
-			}
-		}
-	}
-	
-#ifdef  DEBUG
-	kik_warn_printf( KIK_DEBUG_TAG " XftFontOpen(%s) failed.\n" , fontname) ;
-#endif
-
-	return  0 ;
-
-font_found:
-
-	if( weight == XFT_WEIGHT_BOLD &&
-		XftPatternGetInteger( xfont->pattern , XFT_WEIGHT , 0 , &weight) == XftResultMatch &&
-		weight != XFT_WEIGHT_BOLD)
-	{
-		font->is_double_drawing = 1 ;
-	}
-	else
-	{
-		font->is_double_drawing = 0 ;
-	}
-
-	font->xft_font = xfont ;
-
-	font->height = xfont->height ;
-	font->height_to_baseline = xfont->ascent ;
-
-	font->x_off = 0 ;
-	font->width = ch_width ;
-
-	font->is_proportional = font->is_var_col_width ;
-	
-	/*
-	 * checking if font height/height_to_baseline member is sane.
-	 * font width must be always sane.
-	 */
-	 
-	if( font->height == 0)
-	{
-		/* XXX this may be inaccurate. */
-		font->height = fontsize ;
-	}
-
-	if( font->height_to_baseline == 0)
-	{
-		/* XXX this may be inaccurate. */
-		font->height_to_baseline = fontsize ;
-	}
-
 	/*
 	 * set_decsp_font() is called after dummy font is loaded to get font metrics.
 	 * Since dummy font encoding is "iso8859-1", loading rarely fails.
@@ -1123,21 +1136,7 @@ font_found:
 	return  1 ;
 }
 
-#else
-
-static int
-set_xft_font(
-	x_font_t *  font ,
-	char *  fontname ,
-	u_int  fontsize ,
-	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set */
-	int  use_medium_for_bold
-	)
-{
-	return  set_xfont( font , fontname , fontsize , col_width , use_medium_for_bold) ;
-}
-
-#endif
+#endif	/* USE_TYPE_XCORE */
 
 
 /* --- global functions --- */
@@ -1155,6 +1154,7 @@ x_font_t *
 x_font_new(
 	Display *  display ,
 	ml_font_t  id ,
+	x_type_engine_t  type_engine ,
 	x_font_present_t  font_present ,
 	char *  fontname ,
 	u_int  fontsize ,
@@ -1202,30 +1202,48 @@ x_font_new(
 	{
 		font->is_vertical = 0 ;
 	}
-	
+
+#ifdef  USE_TYPE_XCORE
 	font->xfont = NULL ;
-#ifdef  ANTI_ALIAS
+#endif
+#ifdef  USE_TYPE_XFT
 	font->xft_font = NULL ;
 #endif
 	font->decsp_font = NULL ;
 
-	if( font_present & FONT_AA)
+	switch( type_engine)
 	{
-		if( ! set_xft_font( font , fontname , fontsize , col_width , use_medium_for_bold))
+	default:
+		return  NULL ;
+
+#ifdef  USE_TYPE_XFT
+	case  TYPE_XFT:
+		if( ! set_xft_font( font , fontname , fontsize , col_width , use_medium_for_bold ,
+			(font_present & FONT_AA) == FONT_AA))
 		{
 			free( font) ;
 
 			return  NULL ;
 		}
-	}
-	else
-	{
-		if( ! set_xfont( font , fontname , fontsize , col_width , use_medium_for_bold))
+
+		break ;
+#endif
+
+#ifdef  USE_TYPE_XCORE
+	case  TYPE_XCORE:
+		if( font_present & FONT_AA)
+		{
+			return  NULL ;
+		}
+		else if( ! set_xfont( font , fontname , fontsize , col_width , use_medium_for_bold))
 		{
 			free( font) ;
 
 			return  NULL ;
 		}
+
+		break ;
+#endif
 	}
 
 	return  font ;
@@ -1236,19 +1254,20 @@ x_font_delete(
 	x_font_t *  font
 	)
 {
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 	if( font->xft_font)
 	{
 		XftFontClose( font->display , font->xft_font) ;
 		font->xft_font = NULL ;
 	}
-	else
 #endif
+#ifdef  USE_TYPE_XCORE
 	if( font->xfont)
 	{
 		XFreeFont( font->display , font->xfont) ;
 		font->xfont = NULL ;
 	}
+#endif
 
 	if( font->decsp_font)
 	{
@@ -1282,7 +1301,7 @@ x_change_font_cols(
 	{
 		font->cols = cols ;
 	}
-	
+
 	return  1 ;
 }
 
@@ -1294,13 +1313,9 @@ x_calculate_char_width(
 	mkf_charset_t  cs
 	)
 {
-	if( ! font->is_var_col_width || font->decsp_font)
+	if( font->is_var_col_width && ! font->decsp_font)
 	{
-		return  font->width ;
-	}
-	else
-	{
-	#ifdef  ANTI_ALIAS
+	#ifdef  USE_TYPE_XFT
 		if( font->xft_font)
 		{
 			u_char  ucs4[4] ;
@@ -1318,12 +1333,30 @@ x_calculate_char_width(
 
 			return  xft_calculate_char_width( font->display , font->xft_font , ch , len) ;
 		}
-		else
 	#endif
+	#ifdef  USE_TYPE_XCORE
+		if( font->xfont)
 		{
 			return  calculate_char_width( font->display , font->xfont , ch , len) ;
 		}
+	#endif
+
+		kik_error_printf( __FUNCTION__ " couldn't calculate correct font width.\n") ;
 	}
+
+	return  font->width ;
+}
+
+char **
+x_font_get_cs_names(
+	mkf_charset_t  cs
+	)
+{
+	cs_info_t  * info ;
+
+	info = get_cs_info( cs) ;
+
+	return  info->encoding_names ;
 }
 
 #ifdef  DEBUG
@@ -1333,10 +1366,13 @@ x_font_dump(
 	x_font_t *  font
 	)
 {
+#ifdef  USE_TYPE_XCORE
 	kik_msg_printf( "  id %x: XFont %p" , font->id , font->xfont) ;
-#ifdef  ANTI_ALIAS
+#endif
+#ifdef  USE_TYPE_XFT
 	kik_msg_printf( " XftFont %p" , font->id , font->xft_font) ;
 #endif
+
 	if( font->is_proportional)
 	{
 		kik_msg_printf( " (proportional)") ;
