@@ -1,5 +1,5 @@
 /*
- *	$Id: x_font_manager.c,v 1.17 2003/09/15 13:49:14 arakiken Exp $
+ *	$Id: x_font_manager.c,v 1.22 2005/02/21 06:56:49 h_minami Exp $
  */
 
 #include  "x_font_manager.h"
@@ -80,17 +80,17 @@ change_font_cache(
 }
 
 static int
-release_font_custom(
+release_font_config(
 	x_font_manager_t *  font_man
 	)
 {
-	if( font_man->is_local_font_custom)
+	if( font_man->is_local_font_config)
 	{
-		x_font_custom_delete( font_man->font_custom) ;
+		x_font_config_delete( font_man->font_config) ;
 	}
 	else
 	{
-		x_release_font_custom( font_man->font_custom) ;
+		x_release_font_config( font_man->font_config) ;
 	}
 
 	return  1 ;
@@ -102,6 +102,7 @@ release_font_custom(
 x_font_manager_t *
 x_font_manager_new(
 	Display *  display ,
+	x_type_engine_t  type_engine ,
 	x_font_present_t  font_present ,
 	u_int  font_size ,
 	mkf_charset_t  usascii_font_cs ,
@@ -121,7 +122,7 @@ x_font_manager_new(
 		return  NULL ;
 	}
 
-	if( ( font_man->font_custom = x_acquire_font_custom( font_present)) == NULL)
+	if( ( font_man->font_config = x_acquire_font_config( type_engine , font_present)) == NULL)
 	{
 		free( font_man) ;
 
@@ -129,26 +130,33 @@ x_font_manager_new(
 	}
 
 	if( ( font_man->font_cache = x_acquire_font_cache( display , font_size , usascii_font_cs ,
-					font_man->font_custom , use_multi_col_char)) == NULL)
+					font_man->font_config , use_multi_col_char)) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_acquire_font_cache() failed.\n") ;
 	#endif
 
-		x_release_font_custom( font_man->font_custom) ;
+		x_release_font_config( font_man->font_config) ;
 		free( font_man) ;
 		
 		return  NULL ;
 	}
 
-	font_man->is_local_font_custom = 0 ;
+	font_man->is_local_font_config = 0 ;
 	
 	font_man->prev_cache.font = 0 ;
 	font_man->prev_cache.xfont = NULL ;
 
 	font_man->usascii_font_cs_changable = usascii_font_cs_changable ;
-	font_man->step_in_changing_font_size = step_in_changing_font_size ;
-
+	if(  x_get_max_font_size() - x_get_min_font_size() >= step_in_changing_font_size)
+	{
+		font_man->step_in_changing_font_size = step_in_changing_font_size ;
+	}
+	else
+	{
+		 font_man->step_in_changing_font_size = x_get_max_font_size() - x_get_min_font_size() ;
+	}
+	
 	return  font_man ;
 }
 
@@ -159,7 +167,7 @@ x_font_manager_delete(
 {
 	x_release_font_cache( font_man->font_cache) ;
 
-	release_font_custom( font_man) ;
+	release_font_config( font_man) ;
 	
 	free( font_man) ;
 
@@ -174,7 +182,7 @@ x_get_font(
 {
 	x_font_t *  xfont ;
 
-	if( font_man->prev_cache.font == font)
+	if( font && (font_man->prev_cache.font == font))
 	{
 		return  font_man->prev_cache.xfont ;
 	}
@@ -183,7 +191,7 @@ x_get_font(
 	{
 		font_man->prev_cache.font = font ;
 		font_man->prev_cache.xfont = xfont ;
-		
+
 		return  xfont ;
 	}
 	else
@@ -220,7 +228,7 @@ x_font_manager_usascii_font_cs_changed(
 
 	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
 				font_man->font_cache->font_size , usascii_font_cs ,
-				font_man->font_custom ,
+				font_man->font_config ,
 				font_man->font_cache->use_multi_col_char)) == NULL)
 	{
 		return  0 ;
@@ -237,43 +245,66 @@ x_change_font_present(
 	x_font_present_t  font_present
 	)
 {
-	x_font_custom_t *  font_custom ;
+	x_font_config_t *  font_config ;
 	x_font_cache_t *  font_cache ;
+#define  __HACK__
+#ifdef  __HACK__
+	x_type_engine_t  type_engine ;
 
-	if( font_present == font_man->font_custom->font_present)
+	/* XXX Hack */
+	if( font_present & FONT_AA)
+	{
+		type_engine = TYPE_XFT ;
+	}
+	else
+	{
+		type_engine = TYPE_XCORE ;
+	}
+#endif
+#undef  __HACK__
+
+	if( font_present == font_man->font_config->font_present)
 	{
 		return  1 ;
 	}
-	
-	if( ( font_custom = x_acquire_font_custom( font_present)) == NULL)
+
+	if( ( font_config = x_acquire_font_config( type_engine , font_present)) == NULL)
 	{
-		return  NULL ;
+		return  0 ;
 	}
-	
+
 	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
 				font_man->font_cache->font_size ,
 				font_man->font_cache->usascii_font_cs ,
-				font_custom , font_man->font_cache->use_multi_col_char)) == NULL)
+				font_config , font_man->font_cache->use_multi_col_char)) == NULL)
 	{
-		x_release_font_custom( font_custom) ;
-		
+		x_release_font_config( font_config) ;
+
 		return  0 ;
 	}
 
 	change_font_cache( font_man , font_cache) ;
 
-	release_font_custom( font_man) ;
-	font_man->font_custom = font_custom ;
+	release_font_config( font_man) ;
+	font_man->font_config = font_config ;
 
 	return  1 ;
 }
 
+x_type_engine_t
+x_get_type_engine(
+	x_font_manager_t *  font_man
+	)
+{
+	return  font_man->font_config->type_engine ;
+}
+
 x_font_present_t
 x_get_font_present(
 	x_font_manager_t *  font_man
 	)
 {
-	return  font_man->font_custom->font_present ;
+	return  font_man->font_config->font_present ;
 }
 
 int
@@ -298,7 +329,7 @@ x_change_font_size(
 
 	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
 				font_size , font_man->font_cache->usascii_font_cs , 
-				font_man->font_custom ,
+				font_man->font_config ,
 				font_man->font_cache->use_multi_col_char)) == NULL)
 	{
 		return  0 ;
@@ -328,7 +359,7 @@ x_larger_font(
 	
 	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
 				font_size , font_man->font_cache->usascii_font_cs , 
-				font_man->font_custom ,
+				font_man->font_config ,
 				font_man->font_cache->use_multi_col_char)) == NULL)
 	{
 		return  0 ;
@@ -358,7 +389,7 @@ x_smaller_font(
 	
 	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
 				font_size , font_man->font_cache->usascii_font_cs , 
-				font_man->font_custom ,
+				font_man->font_config ,
 				font_man->font_cache->use_multi_col_char)) == NULL)
 	{
 		return  0 ;
@@ -393,7 +424,7 @@ x_set_multi_col_char_flag(
 	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
 				font_man->font_cache->font_size ,
 				font_man->font_cache->usascii_font_cs ,
-				font_man->font_custom , flag)) == NULL)
+				font_man->font_config , flag)) == NULL)
 	{
 		return  0 ;
 	}
@@ -484,9 +515,9 @@ x_get_usascii_font_cs(
 }
 
 int
-x_activate_local_font_custom(
+x_activate_local_font_config(
 	x_font_manager_t *  font_man ,
-	x_font_custom_t *  font_custom		/* Should be generated by x_font_custom_new() */
+	x_font_config_t *  font_config		/* Should be generated by x_font_config_new() */
 	)
 {
 	x_font_cache_t *  font_cache ;
@@ -494,34 +525,35 @@ x_activate_local_font_custom(
 	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
 				font_man->font_cache->font_size ,
 				font_man->font_cache->usascii_font_cs ,
-				font_custom , font_man->font_cache->use_multi_col_char)) == NULL)
+				font_config , font_man->font_cache->use_multi_col_char)) == NULL)
 	{
 		return  0 ;
 	}
 
 	change_font_cache( font_man , font_cache) ;
 	
-	release_font_custom( font_man) ;
-	font_man->font_custom = font_custom ;
-	font_man->is_local_font_custom = 1 ;
+	release_font_config( font_man) ;
+	font_man->font_config = font_config ;
+	font_man->is_local_font_config = 1 ;
 		
 	return  1 ;
 }
 
 int
-x_deactivate_local_font_custom(
+x_deactivate_local_font_config(
 	x_font_manager_t *  font_man
 	)
 {
-	x_font_custom_t *  font_custom ;
+	x_font_config_t *  font_config ;
 	x_font_cache_t *  font_cache ;
 
-	if( ! font_man->is_local_font_custom)
+	if( ! font_man->is_local_font_config)
 	{
 		return  0 ;
 	}
 
-	if( ( font_custom = x_acquire_font_custom( font_man->font_custom->font_present)) == NULL)
+	if( ( font_config = x_acquire_font_config( font_man->font_config->type_engine ,
+				font_man->font_config->font_present)) == NULL)
 	{
 		return  0 ;
 	}
@@ -529,18 +561,18 @@ x_deactivate_local_font_custom(
 	if( ( font_cache = x_acquire_font_cache( font_man->font_cache->display ,
 				font_man->font_cache->font_size ,
 				font_man->font_cache->usascii_font_cs ,
-				font_man->font_custom , font_man->font_cache->use_multi_col_char)) == NULL)
+				font_man->font_config , font_man->font_cache->use_multi_col_char)) == NULL)
 	{
-		x_release_font_custom( font_custom) ;
+		x_release_font_config( font_config) ;
 		
 		return  0 ;
 	}
 
 	change_font_cache( font_man , font_cache) ;
 
-	x_font_custom_delete( font_man->font_custom) ;
-	font_man->font_custom = font_custom ;
-	font_man->font_custom = 0 ;
+	x_font_config_delete( font_man->font_config) ;
+	font_man->font_config = font_config ;
+	font_man->font_config = 0 ;
 
 	return  1 ;
 }
