@@ -1,5 +1,5 @@
 /*
- *	$Id: x_font_manager.h,v 1.3 2003/09/13 02:07:46 arakiken Exp $
+ *	$Id: x_font_manager.h,v 1.6 2003/12/27 09:06:22 arakiken Exp $
  */
 
 #ifndef  __X_FONT_MANAGER_H__
@@ -8,9 +8,9 @@
 
 #include  <X11/Xlib.h>
 #include  <kiklib/kik_types.h>
+#include  <ml_char_encoding.h>
 
 #include  "x_font_cache.h"
-#include  "ml_char_encoding.h"
 
 
 typedef struct  x_font_manager
@@ -24,8 +24,8 @@ typedef struct  x_font_manager
 		
 	} prev_cache ;
 
-	x_font_custom_t *  font_custom ;
-	int8_t  is_local_font_custom ;
+	x_font_config_t *  font_config ;
+	int8_t  is_local_font_config ;
 
 	int8_t  usascii_font_cs_changable ;
 	u_int8_t  step_in_changing_font_size ;
@@ -33,7 +33,7 @@ typedef struct  x_font_manager
 }  x_font_manager_t ;
 
 
-x_font_manager_t *  x_font_manager_new( Display *  display ,
+x_font_manager_t *  x_font_manager_new( Display *  display , x_type_engine_t  type_engine ,
 	x_font_present_t  font_present , u_int  font_size , mkf_charset_t  usascii_font_cs ,
 	int  usascii_font_cs_changable , int  use_multi_col_char ,
 	int  step_in_changing_font_size) ;
@@ -49,6 +49,8 @@ int  x_font_manager_usascii_font_cs_changed( x_font_manager_t *  font_man ,
 
 int  x_change_font_present( x_font_manager_t *  font_man , x_font_present_t  font_present) ;
 
+x_type_engine_t  x_get_type_engine( x_font_manager_t *  font_man) ;
+
 x_font_present_t  x_get_font_present( x_font_manager_t *  font_man) ;
 
 int  x_change_font_size( x_font_manager_t *  font_man , u_int  font_size) ;
@@ -67,9 +69,9 @@ XFontSet  x_get_fontset( x_font_manager_t *  font_man) ;
 
 mkf_charset_t  x_get_usascii_font_cs( ml_char_encoding_t  encoding) ;
 
-int  x_activate_local_font_custom( x_font_manager_t *  font_man , x_font_custom_t *  font_custom) ;
+int  x_activate_local_font_config( x_font_manager_t *  font_man , x_font_config_t *  font_config) ;
 
-int  x_deactivate_local_font_custom( x_font_manager_t *  font_man) ;
+int  x_deactivate_local_font_config( x_font_manager_t *  font_man) ;
 
 
 #endif
