@@ -1,9 +1,10 @@
 /*
- *	$Id: x_im.c,v 1.9 2004/10/11 11:09:19 seiichi Exp $
+ *	$Id: x_im.c,v 1.17 2005/02/22 17:00:30 seiichi Exp $
  */
 
 #include  <kiklib/kik_mem.h>	/* malloc/alloca/free */
 #include  <kiklib/kik_str.h>	/* kik_str_alloca_dup kik_str_sep kik_snprintf*/
+#include  <kiklib/kik_locale.h>
 
 #include  "x_im.h"
 #include  "x_term_manager.h"
@@ -19,7 +20,8 @@
 typedef  x_im_t * (*x_im_new_func_t)( u_int64_t  magic ,
 				      ml_char_encoding_t  term_encoding ,
 				      x_im_export_syms_t *  syms ,
-				      char *  engine) ;
+				      char *  engine ,
+				      u_int  mod_ignore_mask) ;
 
 
 /* --- static variables --- */
@@ -45,9 +47,9 @@ static  x_im_export_syms_t  im_export_syms =
 
 } ;
 
-#if  defined(USE_UIM) && defined(USE_M17NLIB)
-static int ref_count_uim = 0 ;
-static int ref_count_m17nlib = 0 ;
+#if  1
+/* restroing locale which was overwritten by SCIM */
+#  define  RESTORE_LOCALE  1
 #endif
 
 /* --- static functions --- */
@@ -60,6 +62,7 @@ dlsym_im_new_func(
 	)
 {
 	char * libname ;
+	char * symname ;
 
 	if( ! im_name)
 	{
@@ -69,7 +72,7 @@ dlsym_im_new_func(
 	if( ! ( libname = alloca( strlen( im_name) + 4)))
 	{
 	#ifdef  DEBUG
-		kik_debug_printf( KIK_DEBUG_TAG " malloc() failed.\n") ;
+		kik_debug_printf( KIK_DEBUG_TAG " alloca() failed.\n") ;
 	#endif
 
 		return  0 ;
@@ -77,13 +80,23 @@ dlsym_im_new_func(
 
 	sprintf( libname , "im-%s" , im_name) ;
 
+	if( ! ( symname = alloca( strlen( im_name) + 8)))
+	{
+	#ifdef  DEBUG
+		kik_debug_printf( KIK_DEBUG_TAG " alloca() failed.\n") ;
+	#endif
+
+		return  0 ;
+	}
+
+	sprintf( symname , "im_%s_new" , im_name) ;
+
 	if( ! ( *handle = kik_dl_open( IM_DIR , libname)))
 	{
 		return  0 ;
 	}
 
-	if( ! ( *func = (x_im_new_func_t) kik_dl_func_symbol( *handle ,
-							      "im_new")))
+	if( ! ( *func = (x_im_new_func_t) kik_dl_func_symbol( *handle , symname)))
 	{
 		kik_dl_close( *handle) ;
 
@@ -100,7 +113,8 @@ x_im_t *
 x_im_new(
 	ml_char_encoding_t  term_encoding ,
 	x_im_event_listener_t *  im_listener ,
-	char *  input_method
+	char *  input_method ,
+	u_int  mod_ignore_mask
 	)
 {
 	x_im_t *  im ;
@@ -108,6 +122,14 @@ x_im_new(
 	kik_dl_handle_t  handle ;
 	char *  im_name ;
 	char *  im_attr ;
+#ifdef  RESTORE_LOCALE
+	char *  cur_locale ;
+#endif
+
+	if( input_method == NULL)
+	{
+		return  NULL ;
+	}
 
 	if( strcmp( input_method , "none") == 0)
 	{
@@ -133,55 +155,48 @@ x_im_new(
 		im_attr = NULL ;
 	}
 
-#if  defined(USE_UIM) && defined(USE_M17NLIB)
-	if( ( strcmp( im_name , "uim") == 0 && ref_count_m17nlib) ||
-	    ( strcmp( im_name , "m17nlib") == 0 && ref_count_uim))
-	{
-		kik_error_printf( "uim and m17nlib cannot be used together.\n");
-		return  NULL ;
-	}
-
-	if( strcmp( im_name , "uim") == 0)
-	{
-		ref_count_uim ++ ;
-	}
-	else if( strcmp( im_name , "m17nlib") == 0)
-	{
-		ref_count_m17nlib ++ ;
-	}
+#ifdef  RESTORE_LOCALE
+	cur_locale = kik_str_alloca_dup( kik_get_locale()) ;
 #endif
 
 	if ( ! dlsym_im_new_func( im_name , &func , &handle))
 	{
+	#ifdef  RESTORE_LOCALE
+		kik_locale_init( cur_locale) ;
+	#endif
+		kik_error_printf( "%s: Could not load.\n" , im_name) ;
 		return  NULL ;
 	}
 
-	if( ( im = (*func)( IM_API_COMPAT_CHECK_MAGIC , term_encoding ,
-			    &im_export_syms , im_attr)))
-	{
-		/*
-		 * initializations for x_im_t
-		 */
-		im->handle = handle ;
-		im->name = strdup( im_name) ;
-		im->listener = im_listener ;
-		im->cand_screen = NULL ;
-		im->stat_screen = NULL ;
-		im->preedit.chars = NULL ;
-		im->preedit.num_of_chars = 0 ;
-		im->preedit.filled_len = 0 ;
-		im->preedit.segment_offset = 0 ;
-		im->preedit.cursor_offset = X_IM_PREEDIT_NOCURSOR ;
+#ifdef  RESTORE_LOCALE
+	kik_locale_init( cur_locale) ;
+#endif
 
-	}
-	else
+	if( ! ( im = (*func)( IM_API_COMPAT_CHECK_MAGIC , term_encoding ,
+			      &im_export_syms , im_attr , mod_ignore_mask)))
 	{
-		kik_error_printf( "Cound not open specified "
-				  "input method(%s).\n" , im_name) ;
-
+		kik_error_printf( "%s: Could not open.\n" , im_name) ;
 		kik_dl_close( handle) ;
+
+		return  NULL ;
 	}
 
+
+	/*
+	 * initializations for x_im_t
+	 */
+	im->handle = handle ;
+	im->name = strdup( im_name) ;
+	im->listener = im_listener ;
+	im->cand_screen = NULL ;
+	im->stat_screen = NULL ;
+	im->preedit.chars = NULL ;
+	im->preedit.num_of_chars = 0 ;
+	im->preedit.filled_len = 0 ;
+	im->preedit.segment_offset = 0 ;
+	im->preedit.cursor_offset = X_IM_PREEDIT_NOCURSOR ;
+
+
 	return  im ;
 }
 
@@ -192,17 +207,6 @@ x_im_delete(
 {
 	kik_dl_handle_t  handle ;
 
-#if  defined(USE_UIM) && defined(USE_M17NLIB)
-	if( strcmp( im->name , "uim") == 0)
-	{
-		ref_count_uim -- ;
-	}
-	else if( strcmp( im->name , "m17nlib") == 0)
-	{
-		ref_count_m17nlib -- ;
-	}
-#endif
-
 	free( im->name) ;
 
 	if( im->cand_screen)
@@ -253,7 +257,8 @@ x_im_redraw_preedit(
 					       im->preedit.segment_offset ,
 					       &x , &y))
 		{
-			if( im->stat_screen && im->cand_screen)
+			if( im->stat_screen &&
+			    (im->cand_screen && im->preedit.filled_len))
 			{
 				(*im->stat_screen->hide)( im->stat_screen) ;
 				(*im->cand_screen->show)( im->cand_screen) ;
@@ -266,7 +271,7 @@ x_im_redraw_preedit(
 				(*im->stat_screen->set_spot)( im->stat_screen ,
 							      x , y) ;
 			}
-			else if( im->cand_screen)
+			else if( im->cand_screen && im->preedit.filled_len)
 			{
 				(*im->cand_screen->show)( im->cand_screen) ;
 				(*im->cand_screen->set_spot)( im->cand_screen ,
@@ -294,7 +299,8 @@ x_im_t *
 x_im_new(
 	ml_char_encoding_t  term_encoding ,
 	x_im_event_listener_t *  im_listener ,
-	char *  input_method
+	char *  input_method ,
+	u_int  mod_ignore_mask
 	)
 {
 	return  NULL ;
