@@ -1,5 +1,5 @@
 /*
- *	$Id: x_im.h,v 1.11 2004/10/11 11:09:20 seiichi Exp $
+ *	$Id: x_im.h,v 1.16 2005/02/22 17:00:30 seiichi Exp $
  */
 
 #ifndef  __X_IM_H__
@@ -12,10 +12,10 @@
 #include  "x_im_candidate_screen.h"
 #include  "x_im_status_screen.h"
 
-#define X_IM_PREEDIT_NOCURSOR -1
+#define X_IM_PREEDIT_NOCURSOR  -1
 
 /*
- * the current preedit information
+ * information for the current preedit string
  */
 typedef struct x_im_preedit
 {
@@ -49,7 +49,7 @@ typedef struct x_im_event_listener
 } x_im_event_listener_t ;
 
 /*
- * a dirty hack to replace -export-dynamic option of libtool
+ * dirty hack to replace -export-dynamic option of libtool
  */
 typedef struct x_im_export_syms
 {
@@ -70,11 +70,12 @@ typedef struct x_im_export_syms
 					     size_t , u_char * , size_t) ;
 	mkf_parser_t * (*ml_parser_new)( ml_char_encoding_t) ;
 	mkf_conv_t * (*ml_conv_new)( ml_char_encoding_t) ;
+
 	x_im_candidate_screen_t *  (*x_im_candidate_screen_new)(
 						x_window_manager_t * ,
 						x_font_manager_t * ,
 						x_color_manager_t * ,
-						int , u_int , int , int) ;
+						int , int , u_int , int , int) ;
 	x_im_status_screen_t *  (*x_im_status_screen_new)(
 						x_window_manager_t * ,
 						x_font_manager_t * ,
@@ -86,7 +87,7 @@ typedef struct x_im_export_syms
 } x_im_export_syms_t ;
 
 /*
- * the input method module object
+ * input method module object
  */
 typedef struct x_im
 {
@@ -112,15 +113,16 @@ typedef struct x_im
 
 x_im_t *  x_im_new( ml_char_encoding_t  term_encoding ,
 		    x_im_event_listener_t *  im_listener ,
-		    char *  input_method) ;
+		    char *  input_method ,
+		    u_int  mod_ignore_mask) ;
 
 void  x_im_delete( x_im_t *  xim) ;
 
 void  x_im_redraw_preedit( x_im_t *  im , int  is_focused) ;
 
-#define  IM_API_VERSION  0x06
+#define  IM_API_VERSION  0x08
 #define  IM_API_COMPAT_CHECK_MAGIC			\
-	 (IM_API_VERSION & 0xff << 28 |			\
+	 ((IM_API_VERSION & 0xff << 28) |		\
 	 ((sizeof( x_im_t) & 0xff) << 20) |		\
 	 ((sizeof( x_im_export_syms_t) & 0xff) << 12) |	\
 	 (sizeof( x_im_candidate_screen_t) & 0xfff))
