@@ -1,6 +1,6 @@
 /** @file
  *  @brief image handling functions using gdk-pixbuf
- *	$Id: x_imagelib_gdk.c,v 1.85 2004/05/07 05:43:53 h_minami Exp $
+ *	$Id: x_imagelib_gdk.c,v 1.88 2005/03/05 07:32:14 seiichi Exp $
  */
 
 #include <math.h>                        /* pow */
@@ -9,12 +9,24 @@
 #include <string.h>                      /* memcpy */
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include <kiklib/kik_types.h> /* u_int32_t/u_int16_t */
+#include <kiklib/kik_types.h> /* u_int32_t/u_int16_t, HAVE_STDINT_H */
 #include <kiklib/kik_unistd.h>
 #include <kiklib/kik_str.h>    /* strdup */
 
 #include "x_imagelib.h"
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
+#ifndef SIZE_MAX
+#ifdef SIZE_T_MAX
+#define SIZE_MAX SIZE_T_MAX
+#else
+#define SIZE_MAX ((size_t)-1)
+#endif
+#endif
+
 #define USE_FS 1
 #define SUCCESS 0
 
@@ -49,7 +61,7 @@ fetch_colormap(
 	Colormap  cmap = DefaultColormap( display, screen) ;
 
 	num_cells = DisplayCells( display , screen) ;
-	*color_list = malloc( num_cells * sizeof(XColor)) ;
+	*color_list = calloc( sizeof(XColor), num_cells) ;
 	if( !*color_list)
 	{
 #ifdef DEBUG
@@ -127,10 +139,8 @@ load_file(
 		kik_warn_printf(KIK_DEBUG_TAG "adding pixbuf to cache(%s)\n", path) ;
 #endif
 		/* get rid of old cache data */
-		if( name) /* may be NULL */
-			free( name) ;
+		free( name) ;
 		name = strdup( path) ;
-
 		if( data)
 		{
 #if GDK_PIXBUF_MAJOR >= 2
@@ -294,7 +304,13 @@ create_cardinals_from_bixbuf(
 	unsigned char *pixel ;
 	int i, j ;
 
-	*cardinal = malloc( (width * height + 2) *4) ;
+	if( !width || !height)
+		return -1;
+
+	if( width > ((SIZE_MAX / 4) -2) / height)
+		return -1; /* integer overflow */
+
+	*cardinal = malloc( ((size_t)width * height + 2) *4) ;
 	if( !(*cardinal))
 		return  -1 ;
 
@@ -356,13 +372,12 @@ closest_color_index(
 	int blue
 	)
 {
-	int  closest ;
+	int  closest = 0 ;
 	int  i ;
 	unsigned long  min = 0xffffff ;
 	unsigned long  diff ;
 	int  diff_r, diff_g, diff_b ;
 
-	/* start seeking from the color last used */
 	for( i = 0 ; i < len ; i++)
 	{
 		/* lazy color-space conversion*/
@@ -832,6 +847,9 @@ pixbuf_to_ximage_truecolor(
 	width = gdk_pixbuf_get_width( pixbuf) ;
 	height = gdk_pixbuf_get_height( pixbuf) ;
 
+	if( !width || !height)
+		return  NULL ;
+
 	r_mask = vinfo[0].red_mask ;
 	g_mask = vinfo[0].green_mask ;
 	b_mask = vinfo[0].blue_mask ;
@@ -851,7 +869,10 @@ pixbuf_to_ximage_truecolor(
 		int r_limit, g_limit, b_limit ;
 		u_int16_t *data ;
 
-		data = (u_int16_t *)malloc( width *  height * 2) ;
+		if( width > (SIZE_MAX / 2) / height)
+			return  NULL ;
+
+		data = (u_int16_t *)malloc( (size_t)width * height * 2) ;
 		if( !data)
 			return  NULL ;
 		image = XCreateImage( display, DefaultVisual( display, screen),
@@ -883,9 +904,13 @@ pixbuf_to_ximage_truecolor(
 	{
 		u_int32_t *  data ;
 
-		data = (u_int32_t *)malloc( width *  height * 4) ;
+		if( width > (SIZE_MAX / 4) / height)
+			return  NULL ;
+
+		data = (u_int32_t *)malloc( (size_t)width *  height * 4) ;
 		if( !data)
 			return  NULL;
+
 		image = XCreateImage( display, DefaultVisual( display, screen),
 				      DefaultDepth( display, screen), ZPixmap, 0,
 				      (char *)data,
@@ -1508,10 +1533,10 @@ x_imagelib_load_file_for_background(
 			gdk_pixbuf_unref( cached_pixbuf) ;
 #endif /*GDK_PIXBUF_MAJOR*/
 		}
-		if( cached_mod)
-			free(cached_mod) ;
-		if( cached_pixbuf_file_path)
-			free(cached_pixbuf_file_path) ;
+
+		free(cached_mod) ;
+		free(cached_pixbuf_file_path) ;
+
 		cached_pixbuf_file_path = strdup( file_path) ;
 
 		if( is_picmod_eq( pic_mod, NULL))
@@ -1620,9 +1645,10 @@ x_imagelib_get_transparent_background(
 			if((cache->cooked != None) && (cache->cooked != cache->root))
 				XFreePixmap( win->display, cache->cooked) ;
 			cache->root = current_root ;
+
 			cache->cooked = None ;
-			if ( cache->pic_mod)
-				free(cache->pic_mod) ;
+
+			free(cache->pic_mod) ;
 			cache->pic_mod = NULL ;
 		}
 	}
@@ -1657,8 +1683,7 @@ x_imagelib_get_transparent_background(
 						     gc,
 						     current_root
 						     ,1) ;
-			if(cache->pic_mod)
-				free(cache->pic_mod) ;
+			free(cache->pic_mod) ;
 			cache->pic_mod = malloc(sizeof(x_picture_modifier_t)) ;
 			memcpy( cache->pic_mod, pic_mod, sizeof(x_picture_modifier_t)) ;
 			modify_pixmap( win->display, win->screen, cache->cooked, pic_mod) ;
