@@ -1,5 +1,5 @@
 /*
- *	$Id: x_sb_view_factory.h,v 1.3 2003/06/29 15:00:43 seiichi Exp $
+ *	$Id: x_sb_view_factory.h,v 1.4 2003/10/11 16:27:54 arakiken Exp $
  */
 
 #ifndef  __X_SB_VIEW_FACTORY_H__
@@ -8,8 +8,6 @@
 
 #include  "x_sb_view.h"
 
-typedef  x_sb_view_t * (*x_sb_view_new_func_t)(void) ;
-typedef  x_sb_view_t * (*x_sb_engine_new_func_t)( x_sb_view_conf_t *  conf, int is_transparent) ;
 
 x_sb_view_t *  x_sb_view_new( char *  name) ;
 
