@@ -1,11 +1,10 @@
 /*
- *	$Id: x_screen.h,v 1.18 2003/09/13 02:07:46 arakiken Exp $
+ *	$Id: x_screen.h,v 1.25 2004/10/10 12:48:44 seiichi Exp $
  */
 
 #ifndef  __X_SCREEN_H__
 #define  __X_SCREEN_H__
 
-
 #include  <mkf/mkf_parser.h>
 #include  <kiklib/kik_types.h>		/* u_int/int8_t/size_t */
 #include  <ml_term.h>
@@ -17,8 +16,7 @@
 #include  "x_mod_meta_mode.h"
 #include  "x_bel_mode.h"
 #include  "x_sb_mode.h"
-#include  "x_kbd.h"
-
+#include  "x_im.h"
 
 typedef struct x_screen *  x_screen_ptr_t ;
 
@@ -39,6 +37,8 @@ typedef struct x_system_event_listener
 	ml_term_t *  (*get_pty)( void * , char *) ;
 
 	char *  (*pty_list)( void *) ;
+
+	void  (*config_saved)( void *) ;
 	
 	/* for debug */
 	void  (*exit)( void * , int) ;
@@ -87,15 +87,21 @@ typedef struct  x_screen
 
 	x_sel_event_listener_t  sel_listener ;
 	x_xim_event_listener_t  xim_listener ;
+	x_im_event_listener_t  im_listener ;
 
 	x_shortcut_t *  shortcut ;
 	x_termcap_entry_t *  termcap ;
 
-	x_kbd_t *  kbd ;
+	char *  input_method ;
+	x_im_t *  im ;
+	int  is_preediting ;
+	u_int  im_preedit_beg_row ;
+	u_int  im_preedit_end_row ;
 
 	char *  mod_meta_key ;
 	x_mod_meta_mode_t  mod_meta_mode ;
 	u_int  mod_meta_mask ;
+	u_int  mod_ignore_mask ;
 
 	x_bel_mode_t  bel_mode ;
 
@@ -132,9 +138,9 @@ typedef struct  x_screen
 	u_int8_t  fade_ratio ;
 	int8_t  is_focused ;
 	int8_t  receive_string_via_ucs ;
-	int8_t  xim_open_in_startup ;
 	int8_t  use_vertical_cursor ;
 	int8_t  use_extended_scroll_shortcut ;
+	int8_t  borderless ;
 
 } x_screen_t ;
 
@@ -144,11 +150,12 @@ x_screen_t *  x_screen_new( ml_term_t *  term , x_font_manager_t *  font_man ,
 	u_int  brightness , u_int contrast , u_int gamma ,
 	u_int  fade_ratio , x_shortcut_t *  shortcut ,
 	u_int  screen_width_ratio , u_int  screen_height_ratio ,
-	int  xim_open_in_startup , char *  mod_meta_key , x_mod_meta_mode_t  mod_meta_mode ,
+	char *  mod_meta_key , x_mod_meta_mode_t  mod_meta_mode ,
 	x_bel_mode_t  bel_mode , int  receive_string_via_ucs , char *  pic_file_path ,
 	int  use_transbg , int  use_vertical_cursor , int  big5_buggy ,
 	char *  conf_menu_path_1 , char *  conf_menu_path_2 , char *  conf_menu_path_3 ,
-	int  use_extended_scroll_shortcut , u_int  line_space) ;
+	int  use_extended_scroll_shortcut , int  borderless , u_int  line_space ,
+	char *  input_method) ;
 
 int  x_screen_delete( x_screen_t *  screen) ;
 
