@@ -1,5 +1,5 @@
 /*
- *	$Id: x_shortcut.c,v 1.11 2004/10/10 12:48:44 seiichi Exp $
+ *	$Id: x_shortcut.c,v 1.12 2004/12/13 09:31:55 h_minami Exp $
  */
 
 #include  "x_shortcut.h"
@@ -127,7 +127,7 @@ parse(
 
 		while( *oper != '"' && *oper != '\0')
 		{
-			int  digit ;
+			u_int  digit ;
 
 			if( sscanf( oper , "\\x%2x" , &digit) == 1)
 			{
@@ -159,7 +159,7 @@ parse(
 					}
 					else if( *oper == 'e')
 					{
-						*p = '\e' ;
+						*p = '\033' ;
 					}
 					else
 					{
