@@ -1,5 +1,5 @@
 /*
- *	$Id: x_shortcut.h,v 1.8 2004/10/20 18:46:53 seiichi Exp $
+ *	$Id: x_shortcut.h,v 1.9 2004/12/13 09:31:55 h_minami Exp $
  */
 
 /*
@@ -29,7 +29,7 @@ typedef enum  x_key_func
 	INSERT_SELECTION ,
 	EXIT_PROGRAM ,
 	
-	MAX_KEY_MAPS ,
+	MAX_KEY_MAPS
 
 } x_key_func_t ;
 
