@@ -1,5 +1,5 @@
 /*
- *	$Id: x_term_manager.c,v 1.85 2004/10/23 16:24:26 seiichi Exp $
+ *	$Id: x_term_manager.c,v 1.91 2005/02/28 00:09:16 seiichi Exp $
  */
 
 #include  "x_term_manager.h"
@@ -160,12 +160,17 @@ create_term_intern(void)
 	{
 		ml_term_set_window_name( term , main_config.title) ;
 	}
-	
+
 	if( main_config.icon_name)
 	{
 		ml_term_set_icon_name( term , main_config.icon_name) ;
 	}
 
+	if( main_config.icon_path)
+	{
+		ml_term_set_icon_path( term , main_config.icon_path) ;
+	}
+
 	if( main_config.logging_vt_seq)
 	{
 		ml_term_enable_logging_vt_seq( term) ;
@@ -470,34 +475,12 @@ open_screen_intern(
 
 	if( main_config.icon_path)
 	{
-		if( !(disp->win_man.icon) && !(disp->win_man.mask) && !(disp->win_man.cardinal))
-		{
-			x_window_t  dummy ;
-			int  icon_size = 48 ;
-			x_imagelib_load_file( disp->display , main_config.icon_path,
-					      &(disp->win_man.cardinal),
-					      &(disp->win_man.icon),
-					      &(disp->win_man.mask),
-					      &icon_size ,&icon_size) ;
-
-			dummy.my_window = disp->win_man.group_leader ;
-			dummy.display = disp->win_man.display ;
-			x_window_set_icon( &dummy,
-					   disp->win_man.icon,
-					   disp->win_man.mask,
-					   disp->win_man.cardinal) ;
-		}
-		x_window_set_icon( root,
-				   disp->win_man.icon,
-				   disp->win_man.mask,
-				   disp->win_man.cardinal) ;
-
+		x_window_manager_set_icon( root, main_config.icon_path);
 	}
 
 	if( pty && main_config.cmd_argv)
 	{
 		int  count ;
-		
 		for( count = 0 ; main_config.cmd_argv[count] ; count ++)
 		{
 			ml_term_write( term , main_config.cmd_argv[count] ,
@@ -1051,7 +1034,7 @@ client_connected(void)
 
 	argc = 0 ;
 
-	if( ( argv = alloca( sizeof( char*) * line_len)) == NULL)
+	if( ( argv = alloca( sizeof( char*) * ( line_len + 1))) == NULL)
 	{
 		goto  error ;
 	}
@@ -1124,8 +1107,12 @@ client_connected(void)
 		argv[argc ++] = args_dup ;
 		args_dup = p ;
 	}
+
 parse_end:
 
+	/* NULL terminator (POSIX exec family style) */
+	argv[argc] = NULL ;
+
 #ifdef  __DEBUG
 	{
 		int  i ;
@@ -1148,11 +1135,11 @@ parse_end:
 		/*
 		 * mlclient -P or mlclient --ptylist
 		 */
-		
+
 		ml_term_t **  terms ;
 		u_int  num ;
 		int  count ;
-		
+	
 		num = ml_get_all_terms( &terms) ;
 		for( count = 0 ; count < num ; count ++)
 		{
@@ -1171,6 +1158,15 @@ parse_end:
 			}
 		}
 	}
+	else if( argc == 2 && strncmp( argv[1] , "--kill" , 6) == 0)
+	{
+		if( un_file)
+		{
+			unlink( un_file) ;
+		}
+		fprintf( fp, "bye\n") ;
+		exit( 0) ;
+	}
 	else
 	{
 		kik_conf_t *  conf ;
@@ -1704,6 +1700,7 @@ x_term_manager_init(
 	signal( SIGINT , sig_fatal) ;
 	signal( SIGQUIT , sig_fatal) ;
 	signal( SIGTERM , sig_fatal) ;
+	signal( SIGPIPE , SIG_IGN) ;
 
 	ml_term_manager_init() ;
 	
