@@ -1,5 +1,5 @@
 /*
- *	$Id: x_termcap.h,v 1.4 2003/10/11 12:23:05 arakiken Exp $
+ *	$Id: x_termcap.h,v 1.5 2004/12/13 09:31:55 h_minami Exp $
  */
 
 #ifndef  __X_TERMCAP_H__
@@ -16,7 +16,7 @@ typedef enum  x_termcap_str_field
 	ML_HOME ,
 	ML_END ,
 	
-	MAX_TERMCAP_STR_FIELDS ,
+	MAX_TERMCAP_STR_FIELDS
 
 } x_termcap_str_field_t ;
 
@@ -24,7 +24,7 @@ typedef enum  x_termcap_bool_field
 {
 	ML_BCE ,
 
-	MAX_TERMCAP_BOOL_FIELDS ,
+	MAX_TERMCAP_BOOL_FIELDS
 
 } x_termcap_bool_field_t ;
 
