@@ -1,5 +1,5 @@
 /*
- *	$Id: x_window.c,v 1.72 2004/10/13 10:59:41 seiichi Exp $
+ *	$Id: x_window.c,v 1.76 2005/02/21 14:43:43 h_minami Exp $
  */
 
 /*
@@ -37,6 +37,7 @@
 #endif
 #define  XA_TEXT(display)  (XInternAtom( display , "TEXT" , False))
 #define  XA_UTF8_STRING(display)  (XInternAtom(display , "UTF8_STRING" , False))
+#define  XA_NONE(display)  (XInternAtom(display , "NONE" , False))
 #define  XA_SELECTION(display) (XInternAtom(display , "MLTERM_SELECTION" , False))
 #define  XA_DELETE_WINDOW(display) (XInternAtom(display , "WM_DELETE_WINDOW" , False))
 #define  XA_TAKE_FOCUS(display) (XInternAtom(display , "WM_TAKE_FOCUS" , False))
@@ -512,6 +513,10 @@ x_window_init(
 #endif
 	win->app_name = "mlterm" ;
 
+	win->icon_pix = None;
+	win->icon_mask = None;
+	win->icon_card = NULL;
+	
 	win->window_realized = NULL ;
 	win->window_finalized = NULL ;
 	win->window_exposed = NULL ;
@@ -555,10 +560,7 @@ x_window_final(
 		x_window_final( win->children[count]) ;
 	}
 
-	if( win->children)
-	{
-		free( win->children) ;
-	}
+	free( win->children) ;
 
 	x_window_manager_clear_selection( x_get_root_window( win)->win_man , win) ;
 
@@ -576,6 +578,14 @@ x_window_final(
 
 	XDestroyWindow( win->display , win->my_window) ;
 
+	if( win->icon_pix){
+		XFreePixmap( win->display , win->icon_pix) ;
+	}
+	if( win->icon_mask){
+		XFreePixmap( win->display , win->icon_mask) ;
+	}
+	free( win->icon_card) ;
+	
 	x_xic_deactivate( win) ;
 
 	if( win->window_finalized)
@@ -2028,6 +2038,10 @@ x_window_receive_event(
 
 		notify_reparent_to_children( win) ;
 	}
+	else if( event->type == MapNotify)
+	{
+		notify_reparent_to_children( win) ;
+	}
 	else if( event->type == SelectionClear)
 	{
 		if( win->selection_cleared)
@@ -2119,7 +2133,7 @@ x_window_receive_event(
 		xa_utf8_string = XA_UTF8_STRING(win->display) ;
 		xa_selection = XA_SELECTION(win->display) ;
 
-		if( event->xselection.property == None)
+		if( event->xselection.property == None ||  event->xselection.property == XA_NONE(win->display))
 		{
 			/*
 			 * Selection request failed.
@@ -2898,6 +2912,43 @@ x_set_icon_name(
 	return  1 ;
 }
 
+int
+x_window_remove_icon(
+	x_window_t *  win
+	)
+{
+	XWMHints *hints ;
+
+	hints = XGetWMHints( win->display, win->my_window) ;
+
+	if( hints)
+	{
+
+		hints->flags &= ~IconPixmapHint ;
+		hints->flags &= ~IconMaskHint ;
+
+		XSetWMHints( win->display, win->my_window, hints) ;
+		XFree( hints) ;
+	}
+	/* do not use hints->icon_*. they may be shared! */
+	if( win->icon_pix != None)
+	{
+		XFreePixmap( win->display, win->icon_pix);
+		win->icon_pix = None ;
+	}
+	if( win->icon_mask != None)
+	{
+		XFreePixmap( win->display, win->icon_mask);
+		win->icon_mask = None ;
+	}
+	free( win->icon_card);
+	win->icon_card = NULL ;
+
+	XDeleteProperty( win->display, win->my_window,XA_NET_WM_ICON( win->display)) ;
+
+	return  1 ;
+}
+
 int
 x_window_set_icon(
 	x_window_t *  win,
@@ -2906,7 +2957,7 @@ x_window_set_icon(
 	u_int32_t *  cardinal
 	)
 {
-	XWMHints *hints ;
+	XWMHints *  hints = NULL ;
 
 	/* set extended window manager hint's icon */
 	if( cardinal && cardinal[0] && cardinal[1])
@@ -2921,7 +2972,6 @@ x_window_set_icon(
 				 (cardinal[0])*(cardinal[1]) +2) ;
 	}
 	/* set old style window manager hint's icon */
-	hints = NULL ;
 	if (icon || mask)
 	{
 		hints = XGetWMHints( win->display, win->my_window) ;
@@ -2950,6 +3000,32 @@ x_window_set_icon(
 	return 1 ;
 }
 
+int
+x_window_set_icon_from_file(
+	x_window_t *  win,
+	char *  path
+	)
+{
+	int icon_size = 48;
+
+	x_window_remove_icon( win);
+
+	if( !x_imagelib_load_file( win->display ,
+				   path,
+				   &(win->icon_card),
+				   &(win->icon_pix),
+				   &(win->icon_mask),
+				   &icon_size ,&icon_size))
+	{
+		return  0 ;
+	}
+
+	return x_window_set_icon( win,
+				  win->icon_pix,
+				  win->icon_mask,
+				  win->icon_card) ;
+}
+
 int
 x_window_get_visible_geometry(
 	x_window_t *  win ,
