@@ -1,5 +1,5 @@
 /*
- *	$Id: x_xim.h,v 1.1 2002/05/14 12:05:27 arakiken Exp $
+ *	$Id: x_xim.h,v 1.3 2004/06/20 07:54:59 seiichi Exp $
  */
 
 #ifndef  __X_XIM_H__
@@ -7,8 +7,8 @@
 
 
 #include  <mkf/mkf_parser.h>
+#include  <ml_char_encoding.h>
 
-#include  "ml_char_encoding.h"
 #include  "x_window.h"
 
 
@@ -48,5 +48,6 @@ char *  x_get_xim_name( x_window_t *  win) ;
 
 char *  x_get_xim_locale( x_window_t *  win) ;
 
+char *  x_get_default_xim_name(void) ;
 
 #endif
